<?php

namespace Glopal\Connector\Api;

interface CreateRequestDTOInterface
{
    /**
     * Gets shipping address.
     *
     * @return array
     */
    public function getShippingAddress(): array;

    /**
     * Gets customer last name.
     *
     * @return string
     */
    public function getCustomerLastName(): string;

    /**
     * Gets customer email alias.
     *
     * @return string
     */
    public function getCustomerEmailAlias(): ?string;

    /**
     * Gets currency.
     *
     * @return string
     */
    public function getCurrency(): string;

    /**
     * Gets customer email.
     *
     * @return string
     */
    public function getCustomerEmail(): string;

    /**
     * Gets exchange rate.
     *
     * @return float
     */
    public function getExchangeRate(): float;

    /**
     * Gets billing address.
     *
     * @return array
     */
    public function getBillingAddress(): array;

    /**
     * Gets summary.
     *
     * @return array
     */
    public function getSummary(): array;

    /**
     * Is transactional order?
     *
     * @return bool
     */
    public function isTotalCostOrder(): bool;

    /**
     * Gets total cost.
     *
     * @return array
     */
    public function getTotalCost(): array;

    /**
     * Gets shipping.
     *
     * @return array
     */
    public function getShipping(): array;

    /**
     * Gets attributes.
     *
     * @return bool
     */
    public function isSendOriginNotifications(): bool;

    /**
     * Gets store id.
     *
     * @return int
     */
    public function getStoreId(): int;

    /**
     * Gets created at.
     *
     * @return string
     */
    public function getCreatedAt(): string;

    /**
     * Gets items.
     *
     * @return array
     */
    public function getItems(): array;

    /**
     * Gets exchange currency.
     *
     * @return string
     */
    public function getExchangeCurrency(): string;

    /**
     * Gets customer first name.
     *
     * @return string
     */
    public function getCustomerFirstName(): string;

    /**
     * Gets hub address.
     *
     * @return array|null
     */
    public function getHubAddress(): ?array;

    /**
     * Gets discounts.
     *
     * @return array
     */
    public function getDiscounts(): array;

    /**
     * Is parcel forwarding?
     *
     * @return bool
     */
    public function isParcelForwarding(): bool;

    /**
     * Is different currency?
     *
     * @return bool
     */
    public function isDiffCurrency(): bool;

    /**
     * Retrieve data only for Magento totals
     *
     * @return array
     */
    public function getMagentoTotals(): array;

    /**
     * Gets internal note.
     *
     * @return string|null
     */
    public function getInternalNote(): ?string;

    /**
     * Gets customer note.
     *
     * @return string|null
     */
    public function getCustomerNoteNote(): ?string;
}
