<?php

namespace Glopal\Connector\Api\Data\Price;

interface QuoteItemDataInterface
{
    /**
     * Gets original price.
     *
     * @return float
     */
    public function getOriginalPrice(): float;

    /**
     * Sets original price.
     *
     * @param float $originalPrice
     * @return $this
     */
    public function setOriginalPrice(float $originalPrice): self;

    /**
     * Gets base original price.
     *
     * @return float
     */
    public function getBaseOriginalPrice(): float;

    /**
     * Sets base original price.
     *
     * @param float $baseOriginalPrice
     * @return $this
     */
    public function setBaseOriginalPrice(float $baseOriginalPrice): self;

    /**
     * Gets price.
     *
     * @return float
     */
    public function getPrice(): float;

    /**
     * Sets price.
     *
     * @param float $price
     * @return $this
     */
    public function setPrice(float $price): self;

    /**
     * Gets base price.
     *
     * @return float
     */
    public function getBasePrice(): float;

    /**
     * Sets base price.
     *
     * @param float $basePrice
     * @return $this
     */
    public function setBasePrice(float $basePrice): self;

    /**
     * Gets price included tax.
     *
     * @return float
     */
    public function getPriceInclTax(): float;

    /**
     * Sets price included tax.
     *
     * @param float $priceInclTax
     * @return $this
     */
    public function setPriceInclTax(float $priceInclTax): self;

    /**
     * Gets base price included tax.
     *
     * @return float
     */
    public function getBasePriceInclTax(): float;

    /**
     * Sets base price included tax.
     *
     * @param float $basePriceInclTax
     * @return $this
     */
    public function setBasePriceInclTax(float $basePriceInclTax): self;

    /**
     * Gets tax before discount.
     *
     * @return float
     */
    public function getTaxBeforeDiscount(): float;

    /**
     * Sets tax before discount.
     *
     * @param float $taxBeforeDiscount
     * @return $this
     */
    public function setTaxBeforeDiscount(float $taxBeforeDiscount): self;

    /**
     * Gets base tax before discount.
     *
     * @return float
     */
    public function getBaseTaxBeforeDiscount(): float;

    /**
     * Sets base tax before discount.
     *
     * @param float $baseTaxBeforeDiscount
     * @return $this
     */
    public function setBaseTaxBeforeDiscount(float $baseTaxBeforeDiscount): self;

    /**
     * Gets tax amount.
     *
     * @return float
     */
    public function getTaxAmount(): float;

    /**
     * Sets tax amount.
     *
     * @param float $taxAmount
     * @return $this
     */
    public function setTaxAmount(float $taxAmount): self;

    /**
     * Gets base tax amount.
     *
     * @return float
     */
    public function getBaseTaxAmount(): float;

    /**
     * Sets base tax amount.
     *
     * @param float $baseTaxAmount
     * @return $this
     */
    public function setBaseTaxAmount(float $baseTaxAmount): self;

    /**
     * Gets row total.
     *
     * @return float
     */
    public function getRowTotal(): float;

    /**
     * Sets row total.
     *
     * @param float $rowTotal
     * @return $this
     */
    public function setRowTotal(float $rowTotal): self;

    /**
     * Gets base row total.
     *
     * @return float
     */
    public function getBaseRowTotal(): float;

    /**
     * Sets base row total.
     *
     * @param float $baseRowTotal
     * @return $this
     */
    public function setBaseRowTotal(float $baseRowTotal): self;

    /**
     * Gets row total included tax.
     *
     * @return float
     */
    public function getRowTotalInclTax(): float;

    /**
     * Sets row total included tax.
     *
     * @param float $rowTotalInclTax
     * @return $this
     */
    public function setRowTotalInclTax(float $rowTotalInclTax): self;

    /**
     * Gets base row total included tax.
     *
     * @return float
     */
    public function getBaseRowTotalInclTax(): float;

    /**
     * Sets base row total included tax.
     *
     * @param float $baseRowTotalInclTax
     * @return $this
     */
    public function setBaseRowTotalInclTax(float $baseRowTotalInclTax): self;

    /**
     * Gets row total with discount.
     *
     * @return float
     */
    public function getRowTotalWithDiscount(): float;

    /**
     * Sets row total with discount.
     *
     * @param float $rowTotalWithDiscount
     * @return $this
     */
    public function setRowTotalWithDiscount(float $rowTotalWithDiscount): self;

    /**
     * Gets discount amount.
     *
     * @return float
     */
    public function getDiscountAmount(): float;

    /**
     * Sets discount amount.
     *
     * @param float $discountAmount
     * @return $this
     */
    public function setDiscountAmount(float $discountAmount): self;

    /**
     * Gets base discount amount.
     *
     * @return float
     */
    public function getBaseDiscountAmount(): float;

    /**
     * Sets base discount amount.
     *
     * @param float $baseDiscountAmount
     * @return $this
     */
    public function setBaseDiscountAmount(float $baseDiscountAmount): self;

    /**
     * Gets discount tax compensation amount.
     *
     * @return float
     */
    public function getDiscountTaxCompensationAmount(): float;

    /**
     * Sets discount tax compensation amount.
     *
     * @param float $discountTaxCompensationAmount
     * @return $this
     */
    public function setDiscountTaxCompensationAmount(float $discountTaxCompensationAmount): self;

    /**
     * Gets base discount tax compensation amount.
     *
     * @return float
     */
    public function getBaseDiscountTaxCompensationAmount(): float;

    /**
     * Sets base discount tax compensation amount.
     *
     * @param float $baseDiscountTaxCompensationAmount
     * @return $this
     */
    public function setBaseDiscountTaxCompensationAmount(float $baseDiscountTaxCompensationAmount): self;
}
