<?php

namespace Glopal\Connector\Api;

use Glopal\Connector\Api\Data\OrderAttributeInterface;
use Glopal\Connector\Api\Data\OrderAttributeSearchResultsInterface;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;

interface OrderAttributeRepositoryInterface
{
    /**
     * Save Entity data.
     *
     * @param OrderAttributeInterface $entity
     *
     * @return OrderAttributeInterface
     * @throws CouldNotSaveException
     */
    public function save(OrderAttributeInterface $entity): OrderAttributeInterface;

    /**
     * Load Entity data by given Entity Identity.
     *
     * @param string $entityId
     *
     * @return OrderAttributeInterface
     * @throws NoSuchEntityException
     */
    public function getById(string $entityId): OrderAttributeInterface;

    /**
     * Load Entity data by given Order Identity.
     *
     * @param string $orderId
     *
     * @return OrderAttributeInterface|null
     */
    public function getByOrderId(string $orderId): ?OrderAttributeInterface;

    /**
     * Load Entity data collection by given search criteria.
     *
     * @param SearchCriteriaInterface $criteria
     *
     * @return OrderAttributeSearchResultsInterface
     */
    public function getList(SearchCriteriaInterface $criteria): OrderAttributeSearchResultsInterface;

    /**
     * Delete Entity.
     *
     * @param OrderAttributeInterface $entity
     *
     * @return bool
     * @throws CouldNotDeleteException
     */
    public function delete(OrderAttributeInterface $entity): bool;

    /**
     * Delete entity by given id.
     *
     * @param string $entityId
     *
     * @return bool
     * @throws CouldNotDeleteException
     * @throws NoSuchEntityException
     */
    public function deleteById(string $entityId): bool;

    /**
     * Creating empty entity object.
     *
     * @return OrderAttributeInterface
     */
    public function getEmpty(): OrderAttributeInterface;
}
