<?php

namespace Glopal\Connector\Api;

use Glopal\Connector\Api\Data\OrderItemTotalCostInterface;
use Glopal\Connector\Api\Data\OrderItemTotalCostSearchResultsInterface;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;

interface OrderItemTotalCostRepositoryInterface
{
    /**
     * Save entity data
     *
     * @param OrderItemTotalCostInterface $entity
     *
     * @return OrderItemTotalCostInterface
     * @throws CouldNotSaveException
     */
    public function save(OrderItemTotalCostInterface $entity): OrderItemTotalCostInterface;

    /**
     * Load entity data by given entity Identity
     *
     * @param string $entityId
     *
     * @return OrderItemTotalCostInterface
     * @throws NoSuchEntityException
     */
    public function getById(string $entityId): OrderItemTotalCostInterface;

    /**
     * Load entity data collection by given search criteria
     *
     * @param SearchCriteriaInterface $criteria
     *
     * @return OrderItemTotalCostSearchResultsInterface
     */
    public function getList(SearchCriteriaInterface $criteria): OrderItemTotalCostSearchResultsInterface;

    /**
     * Delete Entity.
     *
     * @param OrderItemTotalCostInterface $entity
     *
     * @return bool
     * @throws CouldNotDeleteException
     */
    public function delete(OrderItemTotalCostInterface $entity): bool;

    /**
     * Delete entity by given id.
     *
     * @param string $entityId
     *
     * @return bool
     * @throws CouldNotDeleteException
     * @throws NoSuchEntityException
     */
    public function deleteById(string $entityId): bool;

    /**
     * Return empty entity object.
     *
     * @return OrderItemTotalCostInterface
     */
    public function getEmpty(): OrderItemTotalCostInterface;
}
