<?php

namespace Glopal\Connector\Api;

use Glopal\Connector\Api\Data\OrderTotalCostInterface;
use Glopal\Connector\Api\Data\OrderTotalCostSearchResultsInterface;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;

interface OrderTotalCostRepositoryInterface
{
    /**
     * Save Entity data
     *
     * @param OrderTotalCostInterface $entity
     *
     * @return OrderTotalCostInterface
     * @throws CouldNotSaveException
     */
    public function save(OrderTotalCostInterface $entity): OrderTotalCostInterface;

    /**
     * Load Entity data by given Entity Identity
     *
     * @param string $entityId
     *
     * @return OrderTotalCostInterface
     * @throws NoSuchEntityException
     */
    public function getById(string $entityId): OrderTotalCostInterface;

    /**
     * Load Entity data by given Order id
     *
     * @param string $orderId
     *
     * @return OrderTotalCostInterface
     * @throws NoSuchEntityException
     */
    public function getByOrderId(string $orderId): ?OrderTotalCostInterface;

    /**
     * Load Entity data collection by given search criteria
     *
     * @param SearchCriteriaInterface $criteria
     *
     * @return OrderTotalCostSearchResultsInterface
     */
    public function getList(SearchCriteriaInterface $criteria): OrderTotalCostSearchResultsInterface;

    /**
     * Delete Entity.
     *
     * @param OrderTotalCostInterface $entity
     *
     * @return bool
     * @throws CouldNotDeleteException
     */
    public function delete(OrderTotalCostInterface $entity): bool;

    /**
     * Delete entity by given id.
     *
     * @param string $entityId
     *
     * @return bool
     * @throws CouldNotDeleteException
     * @throws NoSuchEntityException
     */
    public function deleteById(string $entityId): bool;

    /**
     * Return empty entity object.
     *
     * @return OrderTotalCostInterface
     */
    public function getEmpty(): OrderTotalCostInterface;
}
