<?php

namespace Glopal\Connector\Block\Adminhtml\Form\Field;

use Magento\Config\Block\System\Config\Form\Field\FieldArray\AbstractFieldArray;
use Magento\Framework\DataObject;
use Magento\Framework\Exception\LocalizedException;

class Attributes extends AbstractFieldArray
{
    /** @var GlopalKeySelect */
    private $glopalKeySelectBlock;

    /** @var ProductAttributesSelect */
    private $productAttributeSelectBlock;

    /** @var ProductAttributesSetSelect */
    private $productAttributeSetSelectBlock;

    /**
     * Prepare rendering the new field by adding all the needed columns
     */
    protected function _prepareToRender()
    {
        $this->addColumn('glopal_key', [
            'label' => __('Glopal Attribute'),
            'renderer' => $this->getGlopalKeyBlock()
        ]);

        $this->addColumn('product_attribute_set', [
            'label' => __('Product Attribute Set'),
            'renderer' => $this->getProductAttributeSetBlock()
        ]);

        $this->addColumn('product_attribute', [
            'label' => __('Product Attribute'),
            'renderer' => $this->getProductAttributeBlock()
        ]);

        $this->_addAfter = false;
        $this->_addButtonLabel = __('Add');
    }

    /**
     * Prepare existing row data object
     *
     * @param DataObject $row
     * @throws LocalizedException
     */
    protected function _prepareArrayRow(DataObject $row): void
    {
        $options = [];

        $option = $row->getGlopalKey();
        if ($option !== null) {
            $options['option_' . $this->getGlopalKeyBlock()->calcOptionHash($option)] = 'selected="selected"';
        }

        $option = $row->getProductAttribute();
        if ($option !== null) {
            $options['option_' . $this->getProductAttributeBlock()->calcOptionHash($option)] = 'selected="selected"';
        }

        $option = $row->getProductAttributeSet();
        if ($option !== null) {
            $options['option_' . $this->getProductAttributeSetBlock()->calcOptionHash($option)] = 'selected="selected"';
        }

        $row->setData('option_extra_attrs', $options);
    }

    /**
     * Retrieve Glopal key select block.
     *
     * @return GlopalKeySelect
     * @throws LocalizedException
     */
    private function getGlopalKeyBlock()
    {
        if (!$this->glopalKeySelectBlock) {
            $this->glopalKeySelectBlock = $this->getLayout()->createBlock(
                GlopalKeySelect::class,
                '',
                ['data' => ['is_render_to_js_template' => true]]
            );
        }

        return $this->glopalKeySelectBlock;
    }

    /**
     * Retrieve Glopal product attribute block.
     *
     * @return ProductAttributesSelect
     * @throws LocalizedException
     */
    private function getProductAttributeBlock()
    {
        if (!$this->productAttributeSelectBlock) {
            $this->productAttributeSelectBlock = $this->getLayout()->createBlock(
                ProductAttributesSelect::class,
                '',
                [
                    'data' => [
                        'is_render_to_js_template' => true,
                        'class' => 'product-attributes-dropdown',
                    ]
                ]
            );
        }

        return $this->productAttributeSelectBlock;
    }

    /**
     * Retrieve Glopal product attribute set block.
     *
     * @return ProductAttributesSetSelect
     * @throws LocalizedException
     */
    private function getProductAttributeSetBlock()
    {
        if (!$this->productAttributeSetSelectBlock) {
            $this->productAttributeSetSelectBlock = $this->getLayout()->createBlock(
                ProductAttributesSetSelect::class,
                '',
                [
                    'data' => [
                        'is_render_to_js_template' => true,
                        'class' => 'product-attribute-set-dropdown',
                    ]
                ]
            );
        }

        return $this->productAttributeSetSelectBlock;
    }
}
