<?php

namespace Glopal\Connector\Block\Adminhtml\Order\View\Totals;

use Glopal\Connector\Api\Data\OrderTotalCostInterface;
use Glopal\Connector\ViewModel\TotalsViewModel;
use Magento\Framework\DataObject;
use Magento\Framework\View\Element\Template;
use Magento\Sales\Block\Adminhtml\Order\Totals;

class Duties extends Template
{
    /** @var TotalsViewModel */
    private $totalsViewModel = null;

    /**
     * Initialize duties totals relates with order
     *
     * @return Duties
     */
    public function initTotals(): self
    {
        $this->totalsViewModel = $this->getTotalsViewModel();
        if (null === $this->totalsViewModel) {
            return $this;
        }

        /** @var Totals $parent */
        $parent = $this->getParentBlock();
        $order = $parent->getOrder();

        $totalCost = $this->totalsViewModel->getTotalCost($order);
        if (null === $totalCost) {
            return $this;
        }

        $this->initDutyTotal($totalCost);

        return $this;
    }

    /**
     * Initialize duty total.
     *
     * @param OrderTotalCostInterface $totalCost
     * @return void
     */
    private function initDutyTotal(OrderTotalCostInterface $totalCost): void
    {
        /** @var Totals $parent */
        $parent = $this->getParentBlock();

        if ($this->totalsViewModel->displayBothPrices()) {
            $dutyExclTax = $this->getDutyAmountObject(
                'duty_amount_excl',
                $totalCost->getTotalDutyAmount(),
                $totalCost->getBaseTotalDutyAmount(),
                'Duty (Excl.Tax)'
            );
            $dutyInclTax = $this->getDutyAmountObject(
                'duty_amount_incl',
                $totalCost->getTotalDutyAmount() + $totalCost->getTotalTaxOnDutyAmount(),
                $totalCost->getBaseTotalDutyAmount() + $totalCost->getBaseTotalTaxOnDutyAmount(),
                'Duty (Incl.Tax)'
            );

            $parent->addTotalBefore(
                $dutyInclTax,
                'tax'
            );
            $parent->addTotalBefore(
                $dutyExclTax,
                'duty_amount_incl'
            );

            return;
        }

        if ($this->totalsViewModel->displayPriceInclTax()) {
            $dutyInclTax = $this->getDutyAmountObject(
                'duty_amount',
                $totalCost->getTotalDutyAmount() + $totalCost->getTotalTaxOnDutyAmount(),
                $totalCost->getBaseTotalDutyAmount() + $totalCost->getBaseTotalTaxOnDutyAmount(),
                'Duty'
            );

            $parent->addTotalBefore(
                $dutyInclTax,
                'tax'
            );
        } else {
            $dutyExclTax = $this->getDutyAmountObject(
                'duty_amount',
                $totalCost->getTotalDutyAmount(),
                $totalCost->getBaseTotalDutyAmount(),
                'Duty'
            );

            $parent->addTotalBefore(
                $dutyExclTax,
                'tax'
            );
        }
    }

    /**
     * Retrieve duty amount object.
     *
     * @param string $code
     * @param float $amount
     * @param float $baseAmount
     * @param string $label
     * @return DataObject
     */
    private function getDutyAmountObject(
        string $code,
        float $amount,
        float $baseAmount,
        string $label
    ): DataObject {
        return new DataObject(
            [
                'code' => $code,
                'value' => $amount,
                'base_value' => $baseAmount,
                'label' => __($label),
            ]
        );
    }
}
