<?php

namespace Glopal\Connector\Block\Adminhtml\Order\View\Totals;

use Glopal\Connector\Api\Data\OrderAttributeInterface;
use Glopal\Connector\ViewModel\TotalsViewModel;
use Magento\Framework\DataObject;
use Magento\Framework\View\Element\Template;
use Magento\Sales\Block\Adminhtml\Order\Totals;

class ExtraCharges extends Template
{
    /** @var TotalsViewModel */
    private $totalsViewModel = null;

    /**
     * Initialize extra charges totals relates with order
     *
     * @return ExtraCharges
     */
    public function initTotals(): self
    {
        $this->totalsViewModel = $this->getTotalsViewModel();
        if (null === $this->totalsViewModel) {
            return $this;
        }

        /** @var Totals $parent */
        $parent = $this->getParentBlock();
        $order = $parent->getOrder();

        $orderAttributes = $this->totalsViewModel->getOrderAttributes($order);
        if (null === $orderAttributes) {
            return $this;
        }

        if (empty($orderAttributes->getExtraAmount())) {
            return $this;
        }

        $this->initExtraChargesTotal($orderAttributes);

        return $this;
    }

    /**
     * Initialize extra charges total.
     *
     * @param OrderAttributeInterface $orderAttribute
     * @return void
     */
    private function initExtraChargesTotal(OrderAttributeInterface $orderAttribute): void
    {
        /** @var Totals $parent */
        $parent = $this->getParentBlock();

        if ($this->totalsViewModel->displayBothPrices()) {
            $extraChargesExclTax = $this->getAmountObject(
                'extra_charges_amount_excl',
                $orderAttribute->getExtraAmount(),
                $orderAttribute->getBaseExtraAmount(),
                'Extra Charges (Excl.Tax)'
            );
            $extraChargesInclTax = $this->getAmountObject(
                'extra_charges_amount_incl',
                $orderAttribute->getExtraAmount() + $orderAttribute->getExtraTaxOnAmount(),
                $orderAttribute->getBaseExtraAmount() + $orderAttribute->getBaseExtraTaxOnAmount(),
                'Extra Charges (Incl.Tax)'
            );

            $parent->addTotalBefore(
                $extraChargesInclTax,
                'shipping'
            );
            $parent->addTotalBefore(
                $extraChargesExclTax,
                'extra_charges_amount_incl'
            );

            return;
        }

        if ($this->totalsViewModel->displayPriceInclTax()) {
            $extraChargesInclTax = $this->getAmountObject(
                'extra_charges_amount',
                $orderAttribute->getExtraAmount() + $orderAttribute->getExtraTaxOnAmount(),
                $orderAttribute->getBaseExtraAmount() + $orderAttribute->getBaseExtraTaxOnAmount(),
                'Extra Charges'
            );

            $parent->addTotalBefore(
                $extraChargesInclTax,
                'shipping'
            );
        } else {
            $extraChargesExclTax = $this->getAmountObject(
                'extra_charges_amount',
                $orderAttribute->getExtraAmount(),
                $orderAttribute->getBaseExtraAmount(),
                'Extra Charges'
            );

            $parent->addTotalBefore(
                $extraChargesExclTax,
                'shipping'
            );
        }
    }

    /**
     * Retrieve amount object.
     *
     * @param string $code
     * @param float $amount
     * @param float $baseAmount
     * @param string $label
     * @return DataObject
     */
    private function getAmountObject(
        string $code,
        float $amount,
        float $baseAmount,
        string $label
    ): DataObject {
        return new DataObject(
            [
                'code' => $code,
                'value' => $amount,
                'base_value' => $baseAmount,
                'label' => __($label),
            ]
        );
    }
}
