<?php

namespace Glopal\Connector\Block;

use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Helper\ModuleData;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\View\Element\Template;
use Magento\Framework\View\Element\Template\Context;
use Magento\Store\Api\Data\StoreInterface;

class Head extends Template
{
    /** @var StoreInterface */
    private $store;

    /** @var Configuration */
    private $configuration;

    /** @var ModuleData */
    private $helper;

    /**
     * @param Context $context
     * @param Configuration $configuration
     * @param ModuleData $helper
     * @param array $data
     * @throws NoSuchEntityException
     */
    public function __construct(
        Context $context,
        Configuration $configuration,
        ModuleData $helper,
        array $data = []
    ) {
        parent::__construct($context, $data);

        $this->store = $this->_storeManager->getStore();
        $this->configuration = $configuration;
        $this->helper = $helper;
    }

    /**
     * Is redirector enabled?
     *
     * @return bool
     */
    public function isRedirectorEnabled(): bool
    {
        return $this->configuration->getGeneralFlag('redirector_enabled');
    }

    /**
     * Gets plugin version.
     *
     * @return string
     */
    public function getPluginVersion(): string
    {
        return "magento2-" . $this->helper->getExtensionVersion();
    }

    /**
     * Gets store view id.
     *
     * @return int
     */
    public function getStoreViewId(): int
    {
        return $this->store->getId();
    }

    /**
     * Gets short name.
     *
     * @return string
     */
    public function getShortName(): string
    {
        return $this->configuration->getGeneralValue('short_name');
    }

    /**
     * Gets loader url.
     *
     * @return string
     */
    public function getLoaderUrl(): string
    {
        return $this->configuration->getGeneralValue('remote_assets_url') . '/common/js/webinterpret-loader.js';
    }

    /**
     * Gets head html.
     *
     * @return string
     */
    public function getHeadHtml(): string
    {
        return $this->configuration->getGeneralValue('head');
    }
}
