<?php

namespace Glopal\Connector\Block;

use Magento\Catalog\Block\Product\AbstractProduct;
use Magento\Catalog\Block\Product\Context;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Locale\Resolver;
use Magento\Framework\UrlInterface;
use Magento\Store\Api\Data\StoreInterface;

class Product extends AbstractProduct
{
    /** @var StoreInterface */
    private $store;

    /** @var Resolver */
    private $localeResolver;

    /**
     * @param Context $context
     * @param Resolver $localeResolver
     * @param array $data
     */
    public function __construct(
        Context $context,
        Resolver $localeResolver,
        array $data = []
    ) {
        parent::__construct($context, $data);

        $this->localeResolver = $localeResolver;
    }

    /**
     * Gets locale code.
     *
     * @return string
     */
    public function getLocaleCode(): string
    {
        return $this->localeResolver->getLocale();
    }

    /**
     * Gets store url.
     *
     * @return string
     * @throws NoSuchEntityException
     */
    public function getStoreUrl(): string
    {
        return $this->getStore()->getBaseUrl(UrlInterface::URL_TYPE_WEB);
    }

    /**
     * Gets store view id.
     *
     * @return int
     * @throws NoSuchEntityException
     */
    public function getStoreViewId(): int
    {
        return $this->getStore()->getId();
    }

    /**
     * Get store.
     *
     * @return StoreInterface
     * @throws NoSuchEntityException
     */
    private function getStore(): StoreInterface
    {
        if (!isset($this->store)) {
            $this->store = $this->_storeManager->getStore();
        }

        return $this->store;
    }
}
