<?php

namespace Glopal\Connector\Console;

use Glopal\Connector\Helper\Configuration;

use Magento\Store\Model\ScopeInterface;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;

class OnboardStore extends Command
{
    /** @var Configuration */
    private $configuration;

    /**
     * @param Configuration $configuration
     */
    public function __construct(Configuration $configuration)
    {
        $this->configuration = $configuration;

        parent::__construct();
    }

    /**
     * @inheritdoc
     */
    protected function configure()
    {
        $this->setName('glopal:store:onboard');
        $this->setDescription('Onboard store');
        $this->setDefinition([
             new InputOption(
                 'store-view-id',
                 null,
                 InputOption::VALUE_REQUIRED,
                 'Store View Id'
             ),
             new InputOption(
                 'short-name',
                 null,
                 InputOption::VALUE_REQUIRED,
                 'Glopal Store Id'
             ),
        ]);
    }

    /**
     * @inheritdoc
     */
    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $storeId = (int) $input->getOption('store-view-id');
        $shortName = $input->getOption('short-name');

        $this->configuration->saveGeneralValue(
            'short_name',
            $shortName,
            ScopeInterface::SCOPE_STORES,
            $storeId
        );
        $this->configuration->reinitialiseConfig();

        $newShortName = $this->configuration->getGeneralValue('short_name', $storeId);
        if ($shortName !== $newShortName) {
            $output->writeln("Error updating configuration");
        } else {
            $output->writeln("Connecting {$storeId} with {$newShortName}");
        }

        return 0;
    }
}
