<?php

namespace Glopal\Connector\Controller\Cart;

use Glopal\Connector\Response\CartResponseBuilder;
use Magento\Checkout\Model\Session;
use Magento\Framework\App\Action\HttpGetActionInterface;
use Magento\Framework\Controller\Result\Json;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Quote\Api\Data\CartInterface;

class Index implements HttpGetActionInterface
{
    /** @var Session */
    private $checkoutSession;

    /** @var CartResponseBuilder */
    private $cartResponseBuilder;

    /** @var JsonFactory */
    private $resultJsonFactory;

    /**
     * @param Session $checkoutSession
     * @param CartResponseBuilder $cartResponseBuilder
     * @param JsonFactory $resultJsonFactory
     */
    public function __construct(
        Session $checkoutSession,
        CartResponseBuilder $cartResponseBuilder,
        JsonFactory $resultJsonFactory
    ) {
        $this->checkoutSession = $checkoutSession;
        $this->cartResponseBuilder = $cartResponseBuilder;
        $this->resultJsonFactory = $resultJsonFactory;
    }

    /**
     * Execute index action.
     *
     * @return Json
     * @throws LocalizedException
     * @throws NoSuchEntityException
     */
    public function execute(): Json
    {
        /** @var CartInterface $quote */
        $quote = $this->checkoutSession->getQuote();
        $data = $this->cartResponseBuilder->build($quote);

        return $this->resultJsonFactory->create()->setData($data);
    }
}
