<?php

namespace Glopal\Connector\Controller\Checkout;

use Exception;
use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Response\CartResponseBuilder;
use Glopal\Connector\Service\GlopalCheckout;
use Magento\Checkout\Model\Session;
use Magento\Framework\App\ActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Message\ManagerInterface;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Quote\Model\QuoteIdMaskFactory;
use Psr\Log\LoggerInterface;

/**
 * @SuppressWarnings(PHPMD.AllPurposeAction)
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class Index implements ActionInterface
{
    /** @var Session */
    private $checkoutSession;

    /** @var CartResponseBuilder */
    private $cartResponseBuilder;

    /** @var GlopalCheckout */
    private $checkoutService;

    /** @var SerializerInterface */
    private $serializer;

    /** @var ResultFactory */
    private $resultFactory;

    /** @var LoggerInterface */
    private $logger;

    /** @var Configuration */
    private $configuration;

    /** @var ManagerInterface */
    private $messageManager;

    /** @var QuoteIdMaskFactory */
    private $quoteIdMaskFactory;

    /** @var RequestInterface */
    private $request;

    /** @var CartRepositoryInterface */
    private $cartRepository;

    /**
     * @param Session $checkoutSession
     * @param CartResponseBuilder $cartResponseBuilder
     * @param GlopalCheckout $checkoutService
     * @param SerializerInterface $serializer
     * @param ResultFactory $resultFactory
     * @param LoggerInterface $logger
     * @param Configuration $configuration
     * @param ManagerInterface $messageManager
     * @param QuoteIdMaskFactory $quoteIdMaskFactory
     * @param RequestInterface $request
     * @param CartRepositoryInterface $cartRepository
     * @SuppressWarnings(PHPMD.ExcessiveParameterList)
     */
    public function __construct(
        Session $checkoutSession,
        CartResponseBuilder $cartResponseBuilder,
        GlopalCheckout $checkoutService,
        SerializerInterface $serializer,
        ResultFactory $resultFactory,
        LoggerInterface $logger,
        Configuration $configuration,
        ManagerInterface $messageManager,
        QuoteIdMaskFactory $quoteIdMaskFactory,
        RequestInterface $request,
        CartRepositoryInterface $cartRepository
    ) {
        $this->checkoutSession = $checkoutSession;
        $this->cartResponseBuilder = $cartResponseBuilder;
        $this->checkoutService = $checkoutService;
        $this->serializer = $serializer;
        $this->resultFactory = $resultFactory;
        $this->logger = $logger;
        $this->configuration = $configuration;
        $this->messageManager = $messageManager;
        $this->quoteIdMaskFactory = $quoteIdMaskFactory;
        $this->request = $request;
        $this->cartRepository = $cartRepository;
    }

    /**
     * Execute index action.
     *
     * @return ResultInterface
     */
    public function execute(): ResultInterface
    {
        $redirectUrl = null;
        $defaultRedirectPath = $this->configuration->getCheckoutValue('general/fallback_url');
        $redirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);

        if (!$this->configuration->getCheckoutValue('general/is_redirect_enabled')) {
            $this->messageManager->addErrorMessage('Checkout Error');
            $redirect->setPath($defaultRedirectPath);
        }

        try {
            $quote = $this->getQuote();
            $cartTransferData = $this->cartResponseBuilder->build($quote);
            $requestBody = $this->serializer->serialize($cartTransferData);
            $redirectUrl = $this->checkoutService->requestCartTransfer($requestBody);
        } catch (Exception $e) {
            $this->logger->error('Wrong Glopal checkout link generation', ['exception' => $e]);
        }

        if (null === $redirectUrl) {
            // If something goes wrong -> redirects to Magento default page (for now to Cart)
            $this->messageManager->addErrorMessage('Checkout Error');
            $redirect->setPath($defaultRedirectPath);
        } else {
            $redirect->setUrl($redirectUrl);
        }

        return $redirect;
    }

    /**
     * Gets quote from masked quote id or session
     *
     * @return CartInterface
     * @throws NoSuchEntityException
     * @throws LocalizedException
     */
    private function getQuote(): CartInterface
    {
        $maskedQuoteId = $this->request->getParam('masked_quote_id');

        if (null === $maskedQuoteId) {
            return $this->checkoutSession->getQuote();
        }

        $quoteIdMask = $this->quoteIdMaskFactory->create()->load($maskedQuoteId, 'masked_id');

        return $this->cartRepository->get($quoteIdMask->getQuoteId());
    }
}
