<?php

namespace Glopal\Connector\Controller\Checkout;

use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Helper\Utils;
use Magento\Framework\App\Action\HttpGetActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Controller\ResultFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Result\Page;

class Success implements HttpGetActionInterface
{
    /** @var RequestInterface */
    private $request;

    /** @var Configuration */
    private $configuration;

    /** @var Utils */
    private $utils;

    /** @var ResultFactory */
    private $resultFactory;

    /** @var UrlInterface */
    private $url;

    /**
     * @param RequestInterface $request
     * @param Configuration $configuration
     * @param Utils $utils
     * @param ResultFactory $resultFactory
     * @param UrlInterface $url
     */
    public function __construct(
        RequestInterface $request,
        Configuration $configuration,
        Utils $utils,
        ResultFactory $resultFactory,
        UrlInterface $url
    ) {
        $this->request = $request;
        $this->configuration = $configuration;
        $this->utils = $utils;
        $this->resultFactory = $resultFactory;
        $this->url = $url;
    }

    /**
     * Execute success action.
     *
     * @return Page|ResultInterface
     */
    public function execute()
    {
        if (!$this->configuration->getCheckoutFlag('general/is_clear_local_storage') || !$this->validateHmac()) {
            return $this->resultFactory->create(ResultFactory::TYPE_FORWARD)->forward('noroute');
        }

        return $this->resultFactory->create(ResultFactory::TYPE_PAGE);
    }

    /**
     * Validate requested hmac key.
     *
     * @return bool
     */
    private function validateHmac(): bool
    {
        if (!$this->configuration->getCheckoutFlag('general/success_action_validate_hmac')) {
            return true;
        }

        $requestedHmacKey = $this->request->getParam('sig');
        if (null === $requestedHmacKey) {
            return false;
        }

        $secretKey = $this->configuration->getGeneralValue('key');
        $expectedHmacKey = $this->utils->buildHmacSigFromUrl($this->url->getCurrentUrl(), $secretKey);

        return hash_equals($expectedHmacKey, $requestedHmacKey);
    }
}
