<?php

namespace Glopal\Connector\Controller\Configuration;

use Exception;
use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Helper\ModuleData;
use Magento\Framework\App\ActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Controller\Result\JsonFactory;

/**
 * @SuppressWarnings(PHPMD.AllPurposeAction)
 */
class Save implements ActionInterface
{
    /** @var RequestInterface */
    private $request;

    /** @var Configuration */
    private $configuration;

    /** @var ModuleData */
    private $helper;

    /** @var JsonFactory */
    private $jsonResultFactory;

    /**
     * @param RequestInterface $request
     * @param Configuration $configuration
     * @param ModuleData $data
     * @param JsonFactory $jsonResultFactory
     */
    public function __construct(
        RequestInterface $request,
        Configuration $configuration,
        ModuleData $data,
        JsonFactory $jsonResultFactory
    ) {
        $this->request = $request;
        $this->configuration = $configuration;
        $this->helper = $data;
        $this->jsonResultFactory = $jsonResultFactory;
    }

    /**
     * @inheritdoc
     */
    public function execute()
    {
        $result = [];
        if ($this->configuration->getGeneralFlag('signature_verification')
            && !$this->helper->verifyRequestSignature()
        ) {
            return $this->jsonResultFactory->create()
                ->setData($result)
                ->setHttpResponseCode(403);
        }

        try {
            $this->saveConfig();
            $result = ['status' => 'ok'];
        } catch (Exception $e) {
            $result = [
                'status' => 'error',
                'message' => $e->getMessage()
            ];
        }

        return $this->jsonResultFactory->create()->setData($result);
    }

    /**
     * Save the config.
     *
     * @return void
     */
    private function saveConfig()
    {
        $fields = $this->request->getParam('fields');
        if (!empty($fields) && is_array($fields)) {
            foreach ($fields as $field) {
                $path = $field['path'];
                $scope = $field['scope'];
                $scopeId = $field['scope_id'];
                $value = $field['value'];

                if ($value === '') {
                    $this->configuration->deleteValue($path, $scope, $scopeId);
                } else {
                    $this->configuration->saveValue($path, $value, $scope, $scopeId);
                }
            }

            $this->configuration->reinitialiseConfig();
        }
    }
}
