<?php

namespace Glopal\Connector\Controller\Configuration;

use Exception;
use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Helper\ModuleData;
use Magento\Framework\App\ActionInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use Psr\Log\LoggerInterface;

/**
 * @SuppressWarnings(PHPMD.AllPurposeAction)
 */
class View implements ActionInterface
{
    /** @var Configuration */
    private $configuration;

    /** @var ModuleData */
    private $helper;

    /** @var JsonFactory */
    private $jsonResultFactory;

    /** @var RequestInterface */
    private $request;

    /** @var LoggerInterface  */
    private $logger;

    /**
     * @param Configuration $configuration
     * @param ModuleData $data
     * @param JsonFactory $jsonResultFactory
     * @param RequestInterface $request
     * @param LoggerInterface $logger
     */
    public function __construct(
        Configuration $configuration,
        ModuleData $data,
        JsonFactory $jsonResultFactory,
        RequestInterface $request,
        LoggerInterface $logger
    ) {
        $this->configuration = $configuration;
        $this->helper = $data;
        $this->jsonResultFactory = $jsonResultFactory;
        $this->request = $request;
        $this->logger = $logger;
    }

    /**
     * Execute index action
     */
    public function execute()
    {
        $data = [];
        $result = $this->jsonResultFactory->create();

        if ($this->configuration->getGeneralFlag('signature_verification')
            && !$this->helper->verifyRequestSignature()
        ) {
            return $result->setData($data)->setHttpResponseCode(403);
        }

        try {
            return $result->setData($this->viewConfig());
        } catch (Exception $e) {
            $this->logger->error($e->getMessage(), ['exception' => $e]);

            return $result->setData([
                'error' => true,
                'error_message' => $e->getMessage()
            ])->setHttpResponseCode(500);
        }
    }

    /**
     * View configuration
     *
     * @return array
     */
    private function viewConfig(): array
    {
        $data = [];
        $fields = $this->request->getParam('fields');

        if (!empty($fields) && is_array($fields)) {
            foreach ($fields as $field) {
                $path = $field['path'];
                $scope = $field['scope'];
                $scopeId = $field['scope_id'];

                $field['value'] = $this->configuration->getScopedValue($path, $scope, $scopeId);
                $data[] = $field;
            }
        }

        return $data;
    }
}
