<?php

namespace Glopal\Connector\Controller\V1;

use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Helper\ModuleData;
use Glopal\Connector\Profiler\DriverFactory;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Event\ManagerInterface as EventManager;
use Magento\Framework\Exception\NotFoundException;
use Magento\Framework\Exception\RuntimeException;
use Magento\Sales\Model\OrderRepository;

/**
 * @SuppressWarnings(PHPMD.AllPurposeAction)
 */
class Dispatch extends V1Abstract
{
    /** @var array */
    private const ALLOWED_OBJECTS = [
        'order' => true
    ];

    /** @var string */
    private const DEFAULT_OBJECT_TYPE = 'order';

    /** @var OrderRepository */
    private $orderRepository;

    /** @var EventManager */
    private $eventManager;

    /** @var DriverFactory */
    private $driverFactory;

    /**
     * @param Context $context
     * @param JsonFactory $jsonResultFactory
     * @param Configuration $configuration
     * @param ModuleData $helper
     * @param OrderRepository $orderRepository
     * @param EventManager $eventManager
     * @param DriverFactory $driverFactory
     */
    public function __construct(
        Context $context,
        JsonFactory $jsonResultFactory,
        Configuration $configuration,
        ModuleData $helper,
        OrderRepository $orderRepository,
        EventManager $eventManager,
        DriverFactory $driverFactory
    ) {
        parent::__construct($context, $jsonResultFactory, $configuration, $helper);
        $this->orderRepository = $orderRepository;
        $this->eventManager = $eventManager;
        $this->driverFactory = $driverFactory;
    }

    /**
     * @inheritdoc
     */
    protected function process(): array
    {
        if (!$this->getRequest()->isPost()) {
            throw new NotFoundException(__('Wrong method used'));
        }
        $objectType = $this->getRequest()->getParam('object_type', self::DEFAULT_OBJECT_TYPE);
        if (!isset(self::ALLOWED_OBJECTS[$objectType]) || !self::ALLOWED_OBJECTS[$objectType]) {
            throw new NotFoundException(__('Wrong Object Type Provided'));
        }
        $entityId = $this->getRequest()->getParam('entity_id', null);
        $eventName = $this->getRequest()->getParam('event_name', null);
        if (empty($entityId)) {
            throw new NotFoundException(__('Invalid entity id provided'));
        }
        if (empty($eventName)) {
            throw new NotFoundException(__('Invalid event name provided'));
        }
        $action = 'process' . ucfirst($objectType);
        if (!method_exists($this, $action)) {
            throw new RuntimeException(__('Internal Exception, please contat administrator'));
        }
        $dispatchKey = $this->getRequest()->getParam('dispatchKey', null);
        if (empty($dispatchKey)) {
            return $this->$action($eventName, $entityId);
        }

        return $this->$action($eventName, $entityId, $dispatchKey);
    }
}
