<?php

namespace Glopal\Connector\Helper;

use Exception;
use Psr\Log\LoggerInterface;

class CurlRequest
{
    private const USER_AGENT = 'Glopal-M2-Connector';

    /** @var LoggerInterface */
    private $logger;

    /**
     * @param LoggerInterface $logger
     */
    public function __construct(LoggerInterface $logger)
    {
        $this->logger = $logger;
    }

    /**
     * Send POST request.
     *
     * @param string $url
     * @param string $data
     *
     * @return string|null
     * @throws Exception
     */
    public function sendPostRequest(string $url, string $data): ?string
    {
        // phpcs:disable
        $this->logger->debug($url, ['data' => $data]);

        // \Magento\Framework\HTTP\Client\Curl in Magento < 2.1.14
        // does not support JSON string in POST request body
        // https://github.com/magento/magento2/issues/3489

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_USERAGENT, self::USER_AGENT);
        curl_setopt($ch, CURLOPT_MAXREDIRS, 0);
        curl_setopt($ch, CURLOPT_TIMEOUT, 30);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_FAILONERROR, true);
        curl_setopt($ch, CURLOPT_VERBOSE, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'Content-Type: application/json;charset=UTF-8',
            'Content-Length: ' . strlen($data)
        ]);

        $response = curl_exec($ch);
        $errno = curl_errno($ch);

        if ($errno) {
            $error = curl_error($ch);
            $this->logger->debug($error);

            throw new Exception("Request error: {$error}");
        }

        $status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($status !== 200) {
            $this->logger->debug('HTTP status ' . $status);

            throw new Exception("Request error: HTTP status {$status}");
        }

        curl_close($ch);

        return $response;
        // phpcs:enable
    }
}
