<?php

namespace Glopal\Connector\Helper;

class Utils
{
    /**
     * Change item columns order in admin order view.
     *
     * @param array $columnsWithOrder
     * @param array $columns
     * @return array
     */
    public function changeOrderItemColumnsOrdering(array $columnsWithOrder, array $columns): array
    {
        return array_replace(array_flip($columnsWithOrder), $columns);
    }

    /**
     * Builds hmac key by data and secret.
     *
     * @param string $data
     * @param string $secretKey
     * @return string
     */
    public function buildHmacKey(string $data, string $secretKey): string
    {
        return hash_hmac('sha256', $data, $secretKey);
    }

    /**
     * Builds expected hmac from given url
     *
     * @param string $url  Current url, eg. $_SERVER['REQUEST_URI']
     * @param string $secretKey  Shared secret key
     */
    public function buildHmacSigFromUrl(string $url, string $secretKey): string
    {
        // extract expected "sig" and sort query params
        // phpcs:ignore
        $parsedUrl = parse_url($url);
        // phpcs:ignore
        parse_str($parsedUrl['query'], $queryParams);
        unset($queryParams['sig']);
        ksort($queryParams);

        // calculate over url with sorted params
        $urlToSign = sprintf('%s://%s%s', $parsedUrl['scheme'], $parsedUrl['host'], $parsedUrl['path']);
        if (!empty($queryParams)) {
            $urlToSign .= '?' . http_build_query($queryParams);
        }

        return hash_hmac('sha256', $urlToSign, $secretKey, true);
    }
}
