<?php

namespace Glopal\Connector\Model\Config\Source;

use Magento\Directory\Model\ResourceModel\Country\Collection;
use Magento\Framework\Data\OptionSourceInterface;

class CartTransferStoreCode implements OptionSourceInterface
{
    /** @var array */
    private const DESTINATION_COUNTRY_CODES_MAP = [
        'AD' => 'ca-AD',
        'AE' => 'ar-AE',
        'AL' => 'sq-AL',
        'AR' => 'es-AR',
        'AT' => 'de-AT',
        'AU' => 'en-AU',
        'AX' => 'sv-AX',
        'AZ' => 'az-AZ',
        'BA' => 'bs-BA',
        'BB' => 'en-BB',
        'BE' => 'fr-BE',
        'BG' => 'bg-BG',
        'BH' => 'ar-BH',
        'BM' => 'en-BM',
        'BO' => 'es-BO',
        'BR' => 'pt-BR',
        'BS' => 'en-BS',
        'BY' => 'ru-BY',
        'CA' => 'en-CA',
        'CH' => 'de-CH',
        'CL' => 'es-CL',
        'CN' => 'zh_Hant-CN',
        'CO' => 'es-CO',
        'CR' => 'es-CR',
        'CY' => 'el-CY',
        'CZ' => 'cs-CZ',
        'DE' => 'de-DE',
        'DK' => 'da-DK',
        'DO' => 'es-DO',
        'DZ' => 'ar-DZ',
        'EC' => 'es-EC',
        'EE' => 'et-EE',
        'EG' => 'ar-EG',
        'ES' => 'es-ES',
        'FI' => 'fi-FI',
        'FO' => 'fo-FO',
        'FR' => 'fr-FR',
        'GB' => 'en-GB',
        'GE' => 'ka-GE',
        'GG' => 'en-GG',
        'GH' => 'en-GH',
        'GI' => 'en-GI',
        'GR' => 'el-GR',
        'GT' => 'es-GT',
        'HK' => 'en-HK',
        'HR' => 'hr-HR',
        'HU' => 'hu-HU',
        'ID' => 'id-ID',
        'IE' => 'en-IE',
        'IL' => 'he-IL',
        'IM' => 'en-IM',
        'IN' => 'hi-IN',
        'IS' => 'is-IS',
        'IT' => 'it-IT',
        'JE' => 'en-JE',
        'JM' => 'en-JM',
        'JO' => 'ar-JO',
        'JP' => 'ja-JP',
        'KE' => 'en-KE',
        'KH' => 'km-KH',
        'KR' => 'ko-KR',
        'KW' => 'ar-KW',
        'KY' => 'en-KY',
        'KZ' => 'ru-KZ',
        'LI' => 'de-LI',
        'LT' => 'lt-LT',
        'LU' => 'fr-LU',
        'LV' => 'lv-LV',
        'MA' => 'ar-MA',
        'MC' => 'fr-MC',
        'MD' => 'ro-MD',
        'ME' => 'sr-ME',
        'MK' => 'mk-MK',
        'MN' => 'mn-MN',
        'MT' => 'en-MT',
        'MX' => 'es-MX',
        'MY' => 'en-MY',
        'MZ' => 'pt-MZ',
        'NG' => 'en-NG',
        'NI' => 'es-NI',
        'NL' => 'nl-NL',
        'NO' => 'no-NO',
        'NZ' => 'en-NZ',
        'OM' => 'ar-OM',
        'PA' => 'es-PA',
        'PE' => 'es-PE',
        'PG' => 'en-PG',
        'PH' => 'en-PH',
        'PL' => 'pl-PL',
        'PT' => 'pt-PT',
        'QA' => 'ar-QA',
        'RO' => 'ro-RO',
        'RS' => 'sr-RS',
        'RU' => 'ru-RU',
        'SA' => 'ar-SA',
        'SE' => 'sv-SE',
        'SG' => 'en-SG',
        'SI' => 'sl-SI',
        'SK' => 'sk-SK',
        'SM' => 'it-SM',
        'TH' => 'th-TH',
        'TN' => 'ar-TN',
        'TR' => 'tr-TR',
        'TT' => 'en-TT',
        'TW' => 'zh_Hant-TW',
        'TZ' => 'en-TZ',
        'UA' => 'uk-UA',
        'UG' => 'en-UG',
        'US' => 'en-US',
        'UY' => 'es-UY',
        'VN' => 'vi-VN',
        'ZA' => 'en-ZA',
        'ZM' => '"en-ZM',
    ];

    /** @var Collection */
    private $countryCollection;

    /** @var array */
    private $options;

    /**
     * @param Collection $countryCollection
     */
    public function __construct(Collection $countryCollection)
    {
        $this->countryCollection = $countryCollection;
    }

    /**
     * Cart transfer store code source
     *
     * @return array
     */
    public function toOptionArray(): array
    {
        if (!empty($this->options)) {
            return $this->options;
        }

        $this->options = $this->countryCollection->loadData()
            ->setForegroundCountries('')
            ->toOptionArray(false);
        array_unshift($this->options, ['value' => '', 'label' => __('--Please Select--')]);

        foreach ($this->options as $key => &$option) {
            if (empty($option['value'])) {
                continue;
            }

            $value = $option['value'];
            if (!isset(self::DESTINATION_COUNTRY_CODES_MAP[$value])) {
                unset($this->options[$key]);

                continue;
            }

            $option['value'] = self::DESTINATION_COUNTRY_CODES_MAP[$value];
            $option['label'] = $option['label'] . ' (' . $option['value'] . ')';
        }

        return $this->options;
    }
}
