<?php

namespace Glopal\Connector\Model\Order\Invoice\Total;

use Glopal\Connector\Model\Order\Management;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Sales\Model\Order\Invoice;
use Magento\Sales\Model\Order\Invoice\Total\AbstractTotal;

class ItemsDuty extends AbstractTotal
{
    /** @var Management */
    private $orderManagement;

    /**
     * @param Management $orderManagement
     * @param array $data
     */
    public function __construct(Management $orderManagement, array $data = [])
    {
        parent::__construct($data);

        $this->orderManagement = $orderManagement;
    }

    /**
     * Collect invoice items duty total.
     *
     * @param Invoice $invoice
     * @return $this
     * @throws NoSuchEntityException
     */
    public function collect(Invoice $invoice): ItemsDuty
    {
        $order = $invoice->getOrder();
        $orderTotalCost = $this->orderManagement->getTotalCost($order);

        if (null === $orderTotalCost) {
            return $this;
        }

        $itemsDutyOnAmount = 0.0;
        $itemsBaseDutyOnAmount = 0.0;
        foreach ($order->getItems() as $orderItem) {
            $itemTotalCost = $this->orderManagement->getItemsTotalCostByOrderItemId($orderItem->getId());
            if (null === $itemTotalCost) {
                continue;
            }

            $itemsDutyOnAmount += $itemTotalCost->getDutyAmount();
            $itemsBaseDutyOnAmount += $itemTotalCost->getBaseDutyAmount();
        }

        $invoice->setGrandTotal($invoice->getGrandTotal() + $itemsDutyOnAmount);
        $invoice->setBaseGrandTotal($invoice->getBaseGrandTotal() + $itemsBaseDutyOnAmount);

        return $this;
    }
}
