<?php

namespace Glopal\Connector\Model\Order\Invoice\Total;

use Glopal\Connector\Model\Order\Management;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Sales\Model\Order\Invoice;
use Magento\Sales\Model\Order\Invoice\Total\AbstractTotal;

class ShippingAndExtraChargesDuty extends AbstractTotal
{
    /** @var Management */
    private $orderManagement;

    /**
     * @param Management $orderManagement
     * @param array $data
     */
    public function __construct(Management $orderManagement, array $data = [])
    {
        parent::__construct($data);

        $this->orderManagement = $orderManagement;
    }

    /**
     * Collect invoice shipping and extra charged duty totals.
     *
     * @param Invoice $invoice
     * @return $this
     * @throws NoSuchEntityException
     */
    public function collect(Invoice $invoice): ShippingAndExtraChargesDuty
    {
        $order = $invoice->getOrder();
        $orderAttributes = $this->orderManagement->getOrderAttribute($order);

        if (null === $orderAttributes) {
            return $this;
        }

        $shippingDutyAmount = 0;
        $shippingBaseDutyAmount = 0;
        $extraChargesTotal = 0;
        $baseExtraChargesTotal = 0;

        if (!empty($orderAttributes->getExtraAmount())) {
            $extraChargesTotal = $orderAttributes->getExtraAmount();
            $baseExtraChargesTotal = $orderAttributes->getBaseExtraAmount();
        }

        $orderTotalCost = $this->orderManagement->getTotalCost($order);
        if (null !== $orderTotalCost) {
            $shippingDutyAmount = $orderTotalCost->getShippingDutyAmount();
            $shippingBaseDutyAmount = $orderTotalCost->getBaseShippingDutyAmount();
            $extraChargesTotal += $orderTotalCost->getExtraDutyAmount();
            $baseExtraChargesTotal += $orderTotalCost->getBaseExtraDutyAmount();
        }

        $invoice->setGrandTotal($invoice->getGrandTotal() + $shippingDutyAmount + $extraChargesTotal);
        $invoice->setBaseGrandTotal(
            $invoice->getBaseGrandTotal()
            + $shippingBaseDutyAmount
            + $baseExtraChargesTotal
        );

        return $this;
    }
}
