<?php

namespace Glopal\Connector\Model;

use Glopal\Connector\Api\Data\OrderAttributeInterface;
use Glopal\Connector\Model\ResourceModel\OrderAttribute as ResourceModel;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\Model\AbstractModel;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Registry;
use Magento\Framework\Serialize\SerializerInterface;

class OrderAttribute extends AbstractModel implements OrderAttributeInterface
{
    /** @var SerializerInterface */
    private $serializer;

    /**
     * Constructor.
     *
     * @param Context $context
     * @param Registry $registry
     * @param SerializerInterface $serializer
     * @param AbstractResource|null $resource
     * @param AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        Context $context,
        Registry $registry,
        SerializerInterface $serializer,
        AbstractResource $resource = null,
        AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);

        $this->serializer = $serializer;
    }

    // phpcs:ignore
    protected function _construct()
    {
        $this->_init(ResourceModel::class);
    }

    /**
     * @inheritdoc
     */
    public function getId(): ?int
    {
        return $this->getData(self::ENTITY_ID);
    }

    /**
     * @inheritdoc
     */
    public function setId($id): void
    {
        $this->setData(self::ENTITY_ID, $id);
    }

    /**
     * @inheritdoc
     */
    public function getOrderId(): ?int
    {
        return $this->getData(self::ORDER_ID);
    }

    /**
     * @inheritdoc
     */
    public function setOrderId(int $orderId): void
    {
        $this->setData(self::ORDER_ID, $orderId);
    }

    /**
     * @inheritdoc
     */
    public function getExchangeRate(): ?float
    {
        return $this->getData(self::EXCHANGE_RATE);
    }

    /**
     * @inheritdoc
     */
    public function setExchangeRate(float $rate): void
    {
        $this->setData(self::EXCHANGE_RATE, $rate);
    }

    /**
     * @inheritdoc
     */
    public function getFromCurrency(): ?string
    {
        return $this->getData(self::FROM_CURRENCY);
    }

    /**
     * @inheritdoc
     */
    public function setFromCurrency(string $currency): void
    {
        $this->setData(self::FROM_CURRENCY, $currency);
    }

    /**
     * @inheritdoc
     */
    public function getToCurrency(): ?string
    {
        return $this->getData(self::TO_CURRENCY);
    }

    /**
     * @inheritdoc
     */
    public function setToCurrency(string $currency): void
    {
        $this->setData(self::TO_CURRENCY, $currency);
    }

    /**
     * @inheritdoc
     */
    public function getCustomerOriginEmail(): ?string
    {
        return $this->getData(self::CUSTOMER_ORIGIN_EMAIL);
    }

    /**
     * @inheritdoc
     */
    public function setCustomerOriginEmail(string $email): void
    {
        $this->setData(self::CUSTOMER_ORIGIN_EMAIL, $email);
    }

    /**
     * @inheritdoc
     */
    public function getCustomerShippingAddress(): ?array
    {
        return $this->getData(self::CUSTOMER_SHIPPING_ADDRESS);
    }

    /**
     * @inheritdoc
     */
    public function setCustomerShippingAddress(array $shippingAddress): void
    {
        $this->setData(self::CUSTOMER_SHIPPING_ADDRESS, $shippingAddress);
    }

    /**
     * @inheritdoc
     */
    public function getTransactionId(): ?string
    {
        return $this->getData(self::TRANSACTION_ID);
    }

    /**
     * @inheritdoc
     */
    public function setTransactionId(string $transactionId): void
    {
        $this->setData(self::TRANSACTION_ID, $transactionId);
    }

    /**
     * @inheritdoc
     */
    public function getBaseExtraAmount(): float
    {
        return $this->getData(self::BASE_EXTRA_AMOUNT_NET) ?? 0.0;
    }

    /**
     * @inheritdoc
     */
    public function setBaseExtraAmount(float $amount): void
    {
        $this->setData(self::BASE_EXTRA_AMOUNT_NET, $amount);
    }

    /**
     * @inheritdoc
     */
    public function getExtraAmount(): float
    {
        return $this->getData(self::EXTRA_AMOUNT_NET) ?? 0.0;
    }

    /**
     * @inheritdoc
     */
    public function setExtraAmount(float $amount): void
    {
        $this->setData(self::EXTRA_AMOUNT_NET, $amount);
    }

    /**
     * @inheritdoc
     */
    public function getBaseExtraTaxOnAmount(): float
    {
        return $this->getData(self::BASE_EXTRA_TAX_ON_AMOUNT) ?? 0.0;
    }

    /**
     * @inheritdoc
     */
    public function setBaseExtraTaxOnAmount(float $amount): void
    {
        $this->setData(self::BASE_EXTRA_TAX_ON_AMOUNT, $amount);
    }

    /**
     * @inheritdoc
     */
    public function getExtraTaxOnAmount(): float
    {
        return $this->getData(self::EXTRA_TAX_ON_AMOUNT) ?? 0.0;
    }

    /**
     * @inheritdoc
     */
    public function setExtraTaxOnAmount(float $amount): void
    {
        $this->setData(self::EXTRA_TAX_ON_AMOUNT, $amount);
    }

    /**
     * Converting customer shipping address, summary extra amount to string before save
     *
     * @return OrderAttribute
     */
    public function beforeSave()
    {
        $customerShippingAddress = $this->getData(self::CUSTOMER_SHIPPING_ADDRESS);

        if (!empty($customerShippingAddress) && is_array($customerShippingAddress)) {
            $this->setData(
                self::CUSTOMER_SHIPPING_ADDRESS,
                $this->serializer->serialize($customerShippingAddress)
            );
        }

        return parent::beforeSave();
    }

    /**
     * Converting customer shipping address, summary extra amount to array after load
     *
     * @return OrderAttribute
     */
    public function afterLoad()
    {
        $customerShippingAddressValue = $this->getData(self::CUSTOMER_SHIPPING_ADDRESS);

        if (!empty($customerShippingAddressValue)) {
            $this->setData(
                self::CUSTOMER_SHIPPING_ADDRESS,
                $this->serializer->unserialize($customerShippingAddressValue)
            );
        }

        return parent::afterLoad();
    }
}
