<?php

namespace Glopal\Connector\Model;

use Exception;
use Glopal\Connector\Api\Data\OrderItemTotalCostInterface;
use Glopal\Connector\Api\Data\OrderItemTotalCostSearchResultsInterface;
use Glopal\Connector\Api\Data\OrderItemTotalCostSearchResultsInterfaceFactory;
use Glopal\Connector\Api\OrderItemTotalCostRepositoryInterface;
use Glopal\Connector\Model\ResourceModel\OrderItemTotalCost as ResourceModel;
use Glopal\Connector\Model\ResourceModel\OrderItemTotalCost\CollectionFactory as CollectionFactory;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;

/**
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class OrderItemTotalCostRepository implements OrderItemTotalCostRepositoryInterface
{
    /** @var ResourceModel */
    private $resource;

    /** @var OrderItemTotalCostFactory */
    private $entityFactory;

    /** @var CollectionFactory */
    private $collectionFactory;

    /** @var OrderItemTotalCostSearchResultsInterfaceFactory */
    private $searchResultsFactory;

    /** @var CollectionProcessorInterface */
    private $collectionProcessor;

    /**
     * @param ResourceModel $resource
     * @param OrderItemTotalCostFactory $entityFactory
     * @param CollectionFactory $collectionFactory
     * @param OrderItemTotalCostSearchResultsInterfaceFactory $searchResultsFactory
     * @param CollectionProcessorInterface $collectionProcessor
     */
    public function __construct(
        ResourceModel $resource,
        OrderItemTotalCostFactory $entityFactory,
        CollectionFactory $collectionFactory,
        OrderItemTotalCostSearchResultsInterfaceFactory $searchResultsFactory,
        CollectionProcessorInterface $collectionProcessor
    ) {
        $this->resource = $resource;
        $this->entityFactory = $entityFactory;
        $this->collectionFactory = $collectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->collectionProcessor = $collectionProcessor;
    }

    /**
     * @inheritdoc
     */
    public function save(OrderItemTotalCostInterface $entity): OrderItemTotalCostInterface
    {
        try {
            $this->resource->save($entity);
        } catch (Exception $e) {
            throw new CouldNotSaveException(__($e->getMessage()));
        }

        return $entity;
    }

    /**
     * @inheritdoc
     */
    public function getById(string $entityId): OrderItemTotalCostInterface
    {
        $entity = $this->entityFactory->create();
        $this->resource->load($entity, $entityId);

        if (!$entity->getId()) {
            throw new NoSuchEntityException(__('The entity with the "%1" ID doesn\'t exist.', $entityId));
        }

        return $entity;
    }

    /**
     * @inheritdoc
     */
    public function getList(SearchCriteriaInterface $criteria): OrderItemTotalCostSearchResultsInterface
    {
        $collection = $this->collectionFactory->create();
        $this->collectionProcessor->process($criteria, $collection);

        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);
        $searchResults->setItems($collection->getItems());
        $searchResults->setTotalCount($collection->getSize());

        return $searchResults;
    }

    /**
     * @inheritdoc
     */
    public function delete(OrderItemTotalCostInterface $entity): bool
    {
        try {
            $this->resource->delete($entity);
        } catch (Exception $e) {
            throw new CouldNotDeleteException(__($e->getMessage()));
        }

        return true;
    }

    /**
     * @inheritdoc
     */
    public function deleteById(string $entityId): bool
    {
        return $this->delete($this->getById($entityId));
    }

    /**
     * @inheritdoc
     */
    public function getEmpty(): OrderItemTotalCostInterface
    {
        return $this->entityFactory->create();
    }
}
