<?php

namespace Glopal\Connector\Model;

use Exception;
use Glopal\Connector\Api\Data\OrderTotalCostInterface;
use Glopal\Connector\Api\Data\OrderTotalCostSearchResultsInterface;
use Glopal\Connector\Api\OrderTotalCostRepositoryInterface;
use Glopal\Connector\Model\ResourceModel\OrderTotalCost as ResourceModel;
use Glopal\Connector\Model\ResourceModel\OrderTotalCost\CollectionFactory as CollectionFactory;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Api\SearchCriteriaInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;

class OrderTotalCostRepository implements OrderTotalCostRepositoryInterface
{
    /** @var ResourceModel */
    private $resource;

    /** @var OrderTotalCostFactory */
    private $entityFactory;

    /** @var CollectionFactory */
    private $collectionFactory;

    /** @var OrderTotalCostSearchResultsInterface */
    private $searchResultsFactory;

    /** @var CollectionProcessorInterface */
    private $collectionProcessor;

    /**
     * @param ResourceModel $resource
     * @param OrderTotalCostFactory $entityFactory
     * @param CollectionFactory $collectionFactory
     * @param OrderTotalCostSearchResultsInterface $searchResultsFactory
     * @param CollectionProcessorInterface $collectionProcessor
     */
    public function __construct(
        ResourceModel $resource,
        OrderTotalCostFactory $entityFactory,
        CollectionFactory $collectionFactory,
        OrderTotalCostSearchResultsInterface $searchResultsFactory,
        CollectionProcessorInterface $collectionProcessor
    ) {
        $this->resource = $resource;
        $this->entityFactory = $entityFactory;
        $this->collectionFactory = $collectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->collectionProcessor = $collectionProcessor;
    }

    /**
     * @inheritdoc
     */
    public function save(OrderTotalCostInterface $entity): OrderTotalCostInterface
    {
        try {
            $this->resource->save($entity);
        } catch (Exception $e) {
            throw new CouldNotSaveException(__($e->getMessage()));
        }

        return $entity;
    }

    /**
     * @inheritdoc
     */
    public function getById(string $entityId): OrderTotalCostInterface
    {
        $entity = $this->entityFactory->create();
        $this->resource->load($entity, $entityId);

        if (!$entity->getId()) {
            throw new NoSuchEntityException(__('The entity with the "%1" ID doesn\'t exist.', $entityId));
        }

        return $entity;
    }

    /**
     * @inheritdoc
     */
    public function getByOrderId(string $orderId): ?OrderTotalCostInterface
    {
        $entity = $this->entityFactory->create();
        $this->resource->load($entity, $orderId, 'order_id');

        if (!$entity->getId()) {
            return null;
        }

        return $entity;
    }

    /**
     * @inheritdoc
     */
    public function getList(SearchCriteriaInterface $criteria): OrderTotalCostSearchResultsInterface
    {
        $collection = $this->collectionFactory->create();
        $this->collectionProcessor->process($criteria, $collection);

        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);
        $searchResults->setItems($collection->getItems());
        $searchResults->setTotalCount($collection->getSize());

        return $searchResults;
    }

    /**
     * @inheritdoc
     */
    public function delete(OrderTotalCostInterface $entity): bool
    {
        try {
            $this->resource->delete($entity);
        } catch (Exception $e) {
            throw new CouldNotDeleteException(__($e->getMessage()));
        }

        return true;
    }

    /**
     * @inheritdoc
     */
    public function deleteById(string $entityId): bool
    {
        return $this->delete($this->getById($entityId));
    }

    /**
     * @inheritdoc
     */
    public function getEmpty(): OrderTotalCostInterface
    {
        return $this->entityFactory->create();
    }
}
