<?php

namespace Glopal\Connector\Model\Sales\Order\Address;

use Glopal\Connector\Helper\Order;
use Glopal\Connector\Model\Config\Source\InvoiceShippingAddress;
use Magento\Customer\Model\Address\Config as AddressConfig;
use Magento\Framework\Event\ManagerInterface as EventManager;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\Order\Address;
use Magento\Sales\Model\Order\Address\Renderer as BaseRenderer;

class Renderer extends BaseRenderer
{
    /** @var Order */
    private $orderHelper;

    /** @var OrderRepositoryInterface */
    private $orderRepository;

    /**
     * @param AddressConfig $addressConfig
     * @param EventManager $eventManager
     * @param Order $orderHelper
     * @param OrderRepositoryInterface $orderRepository
     */
    public function __construct(
        AddressConfig $addressConfig,
        EventManager $eventManager,
        Order $orderHelper,
        OrderRepositoryInterface $orderRepository
    ) {
        parent::__construct($addressConfig, $eventManager);

        $this->orderHelper = $orderHelper;
        $this->orderRepository = $orderRepository;
    }

    /**
     * Format address in a specific way
     *
     * @param Address $address
     * @param string $type
     * @return string|null
     */
    public function format(Address $address, $type)
    {
        $order = $this->orderRepository->get($address->getOrder()->getId());
        if (!$this->orderHelper->isGlopalOrder($order)) {
            return parent::format($address, $type);
        }

        $orderStore = $address->getOrder()->getStoreId();
        $invoiceShippingAddress = $this->orderHelper->getInvoiceShippingAddress($orderStore);

        if ($invoiceShippingAddress == InvoiceShippingAddress::HUD_SHIPPING_ADDRESS_VALUE) {
            return parent::format($address, $type);
        }

        return $this->orderHelper->getCustomerOriginShippingAddress($order, 'pdf');
    }
}
