<?php
namespace Glopal\Connector\Model\Sales\Pdf;

use Glopal\Connector\Api\Data\OrderTotalCostInterface;
use Glopal\Connector\Model\Order\Management;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Sales\Model\Order\Pdf\Total\DefaultTotal;
use Magento\Tax\Helper\Data;
use Magento\Tax\Model\Calculation;
use Magento\Tax\Model\ResourceModel\Sales\Order\Tax\CollectionFactory;

class Duty extends DefaultTotal
{
    /** @var Management */
    private $orderManagement;

    /** @var OrderTotalCostInterface */
    private $orderTotalCost;

    /**
     * @param Data $taxHelper
     * @param Calculation $taxCalculation
     * @param CollectionFactory $ordersFactory
     * @param Management $orderManagement
     * @param array $data
     */
    public function __construct(
        Data $taxHelper,
        Calculation $taxCalculation,
        CollectionFactory $ordersFactory,
        Management $orderManagement,
        array $data = []
    ) {
        parent::__construct($taxHelper, $taxCalculation, $ordersFactory, $data);

        $this->orderManagement = $orderManagement;
    }

    /**
     * Get amount.
     *
     * @return float
     */
    public function getAmount(): float
    {
        $orderTotalCost = $this->getTotalCost();
        if (null === $orderTotalCost) {
            return 0;
        }

        return $orderTotalCost->getTotalDutyAmount();
    }

    /**
     * Gets totals for display.
     *
     * @return array|array[]
     */
    public function getTotalsForDisplay(): array
    {
        $totalCost = $this->getTotalCost();
        if (null === $totalCost) {
            return [];
        }

        $dutyExclAmount = $this->getOrder()->formatPriceTxt($totalCost->getTotalDutyAmount());
        $dutyInclAmount = $this->getOrder()
            ->formatPriceTxt($totalCost->getTotalDutyAmount() + $totalCost->getTotalTaxOnDutyAmount());

        $store = $this->getOrder()->getStore();
        $taxHelper = $this->_taxHelper;
        $fontSize = $this->getFontSize() ? $this->getFontSize() : 7;

        if ($taxHelper->displaySalesSubtotalBoth($store)) {
            $totals = [
                [
                    'amount' => $this->getAmountPrefix() . $dutyExclAmount,
                    'label' => __($this->getTitle() . ' (Excl. Tax)') . ':',
                    'font_size' => $fontSize,
                ],
                [
                    'amount' => $this->getAmountPrefix() . $dutyInclAmount,
                    'label' => __($this->getTitle() . ' (Incl. Tax)') . ':',
                    'font_size' => $fontSize
                ],
            ];
        } elseif ($taxHelper->displaySalesSubtotalInclTax($store)) {
            $totals = [
                [
                    'amount' => $this->getAmountPrefix() . $dutyInclAmount,
                    'label' => __($this->getTitle()) . ':',
                    'font_size' => $fontSize,
                ],
            ];
        } else {
            $totals = [
                [
                    'amount' => $this->getAmountPrefix() . $dutyExclAmount,
                    'label' => __($this->getTitle()) . ':',
                    'font_size' => $fontSize,
                ],
            ];
        }

        return $totals;
    }

    /**
     * Get total cost.
     *
     * @return OrderTotalCostInterface|null
     * @throws NoSuchEntityException
     */
    private function getTotalCost(): ?OrderTotalCostInterface
    {
        if ($this->orderTotalCost) {
            return $this->orderTotalCost;
        }

        $order = $this->getOrder();
        $this->orderTotalCost = $this->orderManagement->getTotalCost($order);

        return $this->orderTotalCost;
    }
}
