<?php
namespace Glopal\Connector\Model\Sales\Pdf;

use Glopal\Connector\Model\Order\Management;
use Magento\Sales\Model\Order\Pdf\Total\DefaultTotal;
use Magento\Tax\Helper\Data;
use Magento\Tax\Model\Calculation;
use Magento\Tax\Model\ResourceModel\Sales\Order\Tax\CollectionFactory;

class ExtraCharges extends DefaultTotal
{
    /** @var Management */
    private $orderManagement;

    /** @var array|null */
    private $extraAmounts;

    /**
     * @param Data $taxHelper
     * @param Calculation $taxCalculation
     * @param CollectionFactory $ordersFactory
     * @param Management $orderManagement
     * @param array $data
     */
    public function __construct(
        Data $taxHelper,
        Calculation $taxCalculation,
        CollectionFactory $ordersFactory,
        Management $orderManagement,
        array $data = []
    ) {
        parent::__construct($taxHelper, $taxCalculation, $ordersFactory, $data);

        $this->orderManagement = $orderManagement;
    }

    /**
     * Get amount.
     *
     * @return float
     */
    public function getAmount(): float
    {
        $extraAmounts = $this->getExtraAmounts();
        if (empty($extraAmounts)) {
            return 0;
        }

        return $extraAmounts['amount_net'];
    }

    /**
     * Get totals for display.
     *
     * @return array|array[]
     */
    public function getTotalsForDisplay(): array
    {
        $extraAmounts = $this->getExtraAmounts();
        if (empty($extraAmounts)) {
            return [];
        }

        $extraAmountExclTax = $this->getOrder()->formatPriceTxt($extraAmounts['amount_net']);
        $extraAmountInclTax = $this->getOrder()
            ->formatPriceTxt($extraAmounts['amount_net'] + ($extraAmounts['tax_on_amount'] ?? 0.0));

        $store = $this->getOrder()->getStore();
        $taxHelper = $this->_taxHelper;
        $fontSize = $this->getFontSize() ? $this->getFontSize() : 7;

        if ($taxHelper->displaySalesSubtotalBoth($store)) {
            $totals = [
                [
                    'amount' => $this->getAmountPrefix() . $extraAmountExclTax,
                    'label' => __($this->getTitle() . ' (Excl. Tax)') . ':',
                    'font_size' => $fontSize,
                ],
                [
                    'amount' => $this->getAmountPrefix() . $extraAmountInclTax,
                    'label' => __($this->getTitle() . ' (Incl. Tax)') . ':',
                    'font_size' => $fontSize
                ],
            ];
        } elseif ($taxHelper->displaySalesSubtotalInclTax($store)) {
            $totals = [
                [
                    'amount' => $this->getAmountPrefix() . $extraAmountInclTax,
                    'label' => __($this->getTitle()) . ':',
                    'font_size' => $fontSize,
                ],
            ];
        } else {
            $totals = [
                [
                    'amount' => $this->getAmountPrefix() . $extraAmountExclTax,
                    'label' => __($this->getTitle()) . ':',
                    'font_size' => $fontSize,
                ],
            ];
        }

        return $totals;
    }

    /**
     * Get extra amounts.
     *
     * @return array
     */
    private function getExtraAmounts(): array
    {
        if (null !== $this->extraAmounts) {
            return $this->extraAmounts;
        }

        $order = $this->getOrder();
        $orderAttribute = $this->orderManagement->getOrderAttribute($order);

        if (null === $orderAttribute || empty($orderAttribute->getExtraAmount())) {
            return [];
        }

        $this->extraAmounts = [
            'amount_net' => $orderAttribute->getExtraAmount(),
            'tax_on_amount' => $orderAttribute->getExtraTaxOnAmount()
        ];

        return $this->extraAmounts;
    }
}
