<?php

namespace Glopal\Connector\Observer;

use Magento\Framework\App\RequestInterface;
use Magento\Framework\Event\Observer;
use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;

class SetCurrency implements ObserverInterface
{
    private const X_GLOPAL_SOURCE_CURRENCY = 'X-Glopal-Source-Currency';

    /** @var StoreManagerInterface */
    private $storeManager;

    /** @var RequestInterface */
    private $request;

    /**
     * @param StoreManagerInterface $storeManager
     * @param RequestInterface $request
     */
    public function __construct(StoreManagerInterface $storeManager, RequestInterface $request)
    {
        $this->storeManager = $storeManager;
        $this->request = $request;
    }

    /**
     * Set Glopal requested currency
     *
     * @param Observer $observer
     *
     * @return void
     * @throws LocalizedException|NoSuchEntityException
     *
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function execute(Observer $observer)
    {
        $requestedCurrencyCode = $this->request->getHeader(self::X_GLOPAL_SOURCE_CURRENCY);
        if (empty($requestedCurrencyCode)) {
            return;
        }

        /** @var Store $store */
        $store = $this->storeManager->getStore();
        if ($store->getCurrentCurrencyCode() == $requestedCurrencyCode) {
            return;
        }

        $availableCurrencies = $store->getAvailableCurrencyCodes();
        if (in_array($requestedCurrencyCode, $availableCurrencies)) {
            $store->setCurrentCurrencyCode($requestedCurrencyCode);
        }
    }
}
