<?php

namespace Glopal\Connector\Plugin;

use Glopal\Connector\Model\Order\Management;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Sales\Api\Data\OrderExtensionInterfaceFactory;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;

/**
 * @SuppressWarnings(PHPMD.UnusedFormalParameter)
 */
class OrderRepositoryPlugin
{
    /** @var OrderExtensionInterfaceFactory */
    private $orderExtensionFactory;

    /** @var Management */
    private $orderManagement;

    /**
     * @param OrderExtensionInterfaceFactory $orderExtensionFactory
     * @param Management $orderManagement
     */
    public function __construct(OrderExtensionInterfaceFactory $orderExtensionFactory, Management $orderManagement)
    {
        $this->orderExtensionFactory = $orderExtensionFactory;
        $this->orderManagement = $orderManagement;
    }

    /**
     * Gets Glopal extension attributes to an order.
     *
     * @param OrderRepositoryInterface $subject
     * @param OrderInterface $order
     * @return OrderInterface
     * @throws NoSuchEntityException
     */
    public function afterGet(OrderRepositoryInterface $subject, OrderInterface $order): OrderInterface
    {
        $extensionAttributes = $order->getExtensionAttributes();
        if (null === $extensionAttributes) {
            $extensionAttributes = $this->orderExtensionFactory->create();
        }

        $orderAttribute = $this->orderManagement->getOrderAttribute($order);
        if (null !== $orderAttribute) {
            $extensionAttributes->setGlAttribute($orderAttribute);
        }

        $totalCost = $this->orderManagement->getTotalCost($order);
        if (null !== $totalCost) {
            $extensionAttributes->setGlTcc($totalCost);

            $itemsTotalCost = $this->orderManagement->getItemsTotalCost($totalCost->getId());
            if (!empty($itemsTotalCost)) {
                $extensionAttributes->setGlItemsTcc($itemsTotalCost);
            }
        }

        $order->setExtensionAttributes($extensionAttributes);

        return $order;
    }
}
