<?php

namespace Glopal\Connector\Plugin\Sales\Block\Adminhtml\Order\View;

use Glopal\Connector\Helper\Order;
use Glopal\Connector\Model\Config\Source\InvoiceShippingAddress;
use Magento\Framework\App\RequestInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Block\Adminhtml\Order\View\Info;
use Magento\Sales\Model\Order\Address;

/**
 * @SuppressWarnings(PHPMD.UnusedFormalParameter)
 */
class InfoPlugin
{
    /** @var string */
    private const ORDER_INVOICE_VIEW_PATH = 'order_invoice_view';

    /** @var Order */
    private $orderHelper;

    /** @var RequestInterface */
    private $request;

    /** @var OrderRepositoryInterface */
    private $orderRepository;

    /** @var string */
    private $sourceType = 'order';

    /**
     * @param Order $orderHelper
     * @param RequestInterface $request
     * @param OrderRepositoryInterface $orderRepository
     */
    public function __construct(
        Order $orderHelper,
        RequestInterface $request,
        OrderRepositoryInterface $orderRepository
    ) {
        $this->orderHelper = $orderHelper;
        $this->request = $request;
        $this->orderRepository = $orderRepository;
    }

    /**
     * Adds origin shipping address for Glopal orders
     *
     * @param Info $subject
     * @param null|string $result
     * @param Address $address
     *
     * @return null|string
     */
    public function afterGetFormattedAddress(Info $subject, ?string $result, Address $address): ?string
    {
        if ($address->getAddressType() != 'shipping') {
            return $result;
        }

        $pathName = $this->request->getControllerName() . '_' . $this->request->getActionName();
        if ($pathName == self::ORDER_INVOICE_VIEW_PATH) {
            $this->sourceType = 'invoice';
        }

        $order = $this->loadOrder($address->getOrder());
        if (!$order->getId()) {
            return $result;
        }

        $originCustomerShippingAddress = $this->orderHelper->getCustomerOriginShippingAddress($order);
        if (null === $originCustomerShippingAddress) {
            return $result;
        }

        if ($this->sourceType == 'invoice') {
            $invoiceShippingAddress = $this->orderHelper->getInvoiceShippingAddress($order->getStoreId());
            if ($invoiceShippingAddress == InvoiceShippingAddress::DESTINATION_SHIPPING_ADDRESS_VALUE) {
                return $originCustomerShippingAddress;
            }

            return $result;
        }

        return $result . $this->getOriginAddressHtml($originCustomerShippingAddress);
    }

    /**
     * Load order.
     *
     * @param OrderInterface $order
     * @return OrderInterface
     */
    private function loadOrder(OrderInterface $order): OrderInterface
    {
        if ($this->sourceType == 'order') {
            return $order;
        }

        return $this->orderRepository->get($order->getId());
    }

    /**
     * Get original address html.
     *
     * @param string $originCustomerShippingAddress
     * @return string
     */
    private function getOriginAddressHtml(string $originCustomerShippingAddress): string
    {
        return '<br/><br/>'
            . '<strong>' . __('Origin Shipping Address') . '</strong>'
            . '<br/><br/>'
            . $originCustomerShippingAddress;
    }
}
