<?php

namespace Glopal\Connector\Plugin\Sales\Block\Adminhtml\Order\View;

use Glopal\Connector\Helper\Utils;
use Glopal\Connector\Model\Order\Management;
use Magento\Framework\Exception\LocalizedException;
use Magento\Sales\Block\Adminhtml\Order\View\Items;

class ItemsPlugin
{
    private const ORDER_ITEM_COLUMNS_ORDER = [
        'product',
        'status',
        'price-original',
        'price',
        'ordered-qty',
        'subtotal',
        'duty',
        'duty-percent',
        'tax-amount',
        'tax-percent',
        'discont',
        'total'
    ];

    /** @var Utils */
    private $utilsHelper;

    /** @var Management */
    private $orderManagement;

    /**
     * @param Utils $utilsHelper
     * @param Management $orderManagement
     */
    public function __construct(Utils $utilsHelper, Management $orderManagement)
    {
        $this->utilsHelper = $utilsHelper;
        $this->orderManagement = $orderManagement;
    }

    /**
     * Adding sort order to admin order items columns view.
     *
     * @param Items $subject
     * @param array $result
     * @return array
     * @throws LocalizedException
     */
    public function afterGetColumns(Items $subject, array $result): array
    {
        if (empty($result)) {
            return $result;
        }

        $order = $subject->getOrder();
        $orderTotalCosts = $this->orderManagement->getTotalCost($order);

        if (null === $orderTotalCosts) {
            unset($result['duty']);
            unset($result['duty-percent']);

            return $result;
        }

        return $this->utilsHelper->changeOrderItemColumnsOrdering(
            self::ORDER_ITEM_COLUMNS_ORDER,
            $result
        );
    }
}
