<?php

namespace Glopal\Connector\Plugin\Weee\Block\Adminhtml\Items\Price;

use Glopal\Connector\Api\Data\OrderItemTotalCostInterface;
use Glopal\Connector\Model\Order\Management;
use Magento\Quote\Model\Quote\Item\AbstractItem as QuoteItem;
use Magento\Sales\Model\Order\Creditmemo\Item as CreditmemoItem;
use Magento\Sales\Model\Order\Invoice\Item as InvoiceItem;
use Magento\Sales\Model\Order\Item;
use Magento\Weee\Block\Adminhtml\Items\Price\Renderer;

/**
 * @SuppressWarnings(PHPMD.UnusedFormalParameter)
 */
class AddDutyToTotalAmount
{
    /** @var Management */
    private $orderManagement;

    /** @var OrderItemTotalCostInterface[] */
    private $loadedItemTotalCost = [];

    /**
     * @param Management $orderManagement
     */
    public function __construct(Management $orderManagement)
    {
        $this->orderManagement = $orderManagement;
    }

    /**
     * Return the total amount plus duty amount.
     *
     * @param Renderer $subject
     * @param mixed $result
     * @param Item|QuoteItem|InvoiceItem|CreditmemoItem $item
     *
     * @return float
     */
    public function afterGetTotalAmount(Renderer $subject, $result, $item): float
    {
        $itemTotalCost = $this->getItemTotalCost($item);
        if (null === $itemTotalCost) {
            return $result;
        }

        return $result + $itemTotalCost->getDutyAmount() + $itemTotalCost->getTaxOnDutyAmount();
    }

    /**
     * Return the total amount plus duty amount.
     *
     * @param Renderer $subject
     * @param mixed $result
     * @param Item|QuoteItem|InvoiceItem|CreditmemoItem $item
     *
     * @return float
     */
    public function afterGetBaseTotalAmount(Renderer $subject, $result, $item): float
    {
        $itemTotalCost = $this->getItemTotalCost($item);
        if (null === $itemTotalCost) {
            return $result;
        }

        return $result + $itemTotalCost->getBaseDutyAmount() + $itemTotalCost->getBaseTaxOnDutyAmount();
    }

    /**
     * Get item total cost.
     *
     * @param Item|QuoteItem|InvoiceItem|CreditmemoItem $item
     *
     * @return OrderItemTotalCostInterface
     */
    private function getItemTotalCost($item): ?OrderItemTotalCostInterface
    {
        if ($item instanceof QuoteItem) {
            return null;
        }

        if ($item instanceof Item) {
            $itemId = $item->getId();
        } else {
            $itemId = $item->getOrderItemId();
        }

        if (isset($this->loadedItemTotalCost[$itemId])) {
            return $this->loadedItemTotalCost[$itemId];
        }

        $itemTotalCost = $this->orderManagement->getItemsTotalCostByOrderItemId($itemId);
        if (null === $itemTotalCost) {
            return null;
        }

        $this->loadedItemTotalCost[$itemId] = $itemTotalCost;

        return $this->loadedItemTotalCost[$itemId];
    }
}
