<?php

namespace Glopal\Connector\Processor\AfterOrder;

use Glopal\Connector\Api\CreateRequestDTOInterface;
use Glopal\Connector\Api\Data\OrderAttributeInterface;
use Glopal\Connector\Helper\Order;
use Glopal\Connector\Model\Order\Management;
use Magento\Sales\Api\Data\OrderInterface;

class Attributes implements AfterOrderCreateProcessorInterface
{
    /** @var Management */
    private $orderManagement;

    /** @var Order */
    private $orderHelper;

    /**
     * @param Management $orderManagement
     * @param Order $orderHelper
     */
    public function __construct(Management $orderManagement, Order $orderHelper)
    {
        $this->orderHelper = $orderHelper;
        $this->orderManagement = $orderManagement;
    }

    /**
     * @inheritdoc
     */
    public function process(OrderInterface $order, CreateRequestDTOInterface $createRequestDTO): void
    {
        $exchangeRate = $createRequestDTO->getExchangeRate();
        $orderAttributeData = [
            OrderAttributeInterface::EXCHANGE_RATE => $createRequestDTO->getExchangeRate(),
            OrderAttributeInterface::FROM_CURRENCY => $createRequestDTO->getCurrency(),
            OrderAttributeInterface::TO_CURRENCY => $createRequestDTO->getExchangeCurrency(),
            OrderAttributeInterface::CUSTOMER_ORIGIN_EMAIL => $createRequestDTO->getCustomerEmail()
        ];

        if ($createRequestDTO->isParcelForwarding()) {
            $customerOrigShippingAddress = $this->orderHelper->convertAddressFromRequest(
                $createRequestDTO->getShippingAddress()
            );

            $orderAttributeData[OrderAttributeInterface::CUSTOMER_SHIPPING_ADDRESS] = $customerOrigShippingAddress;
        }

        $extraAmounts = $createRequestDTO->getSummary()['extra'] ?? null;
        if (null !== $extraAmounts && !empty($extraAmounts['amount_net'])) {
            $orderAttributeData[OrderAttributeInterface::EXTRA_AMOUNT_NET] = $extraAmounts['amount_net'];
            $orderAttributeData[OrderAttributeInterface::BASE_EXTRA_AMOUNT_NET]
                = $extraAmounts['amount_net'] / $exchangeRate;
            $orderAttributeData[OrderAttributeInterface::EXTRA_TAX_ON_AMOUNT] = $extraAmounts['tax_on_amount'];
            $orderAttributeData[OrderAttributeInterface::BASE_EXTRA_TAX_ON_AMOUNT]
                = $extraAmounts['tax_on_amount'] / $exchangeRate;
        }

        if ($createRequestDTO->isTotalCostOrder()) {
            $orderAttributeData[OrderAttributeInterface::TRANSACTION_ID]
                = $createRequestDTO->getTotalCost()['transaction_id'];
        }

        $this->orderManagement->addOrderAttribute($order, $orderAttributeData);
    }
}
