<?php

namespace Glopal\Connector\Processor\AfterOrder;

use Glopal\Connector\Api\CreateRequestDTOInterface;
use Glopal\Connector\Helper\Order;
use Magento\Framework\DB\Transaction;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Model\Service\InvoiceService;

class Invoice implements AfterOrderCreateProcessorInterface
{
    /** @var Order */
    private $orderHelper;

    /** @var InvoiceService */
    private $invoiceService;

    /** @var Transaction */
    private $transaction;

    /**
     * @param Order $orderHelper
     * @param InvoiceService $invoiceService
     * @param Transaction $transaction
     */
    public function __construct(
        Order $orderHelper,
        InvoiceService $invoiceService,
        Transaction $transaction
    ) {
        $this->orderHelper = $orderHelper;
        $this->invoiceService = $invoiceService;
        $this->transaction = $transaction;
    }

    /**
     * @inheritdoc
     */
    public function process(OrderInterface $order, CreateRequestDTOInterface $createRequestDTO): void
    {
        if (!$this->orderHelper->isCreateInvoice($createRequestDTO->getStoreId())) {
            return;
        }

        if ($order->canInvoice()) {
            $invoice = $this->invoiceService->prepareInvoice($order)
                ->register()
                ->save();

            $this->transaction->addObject($invoice)
                ->addObject($invoice->getOrder())
                ->save();
        }
    }
}
