<?php

namespace Glopal\Connector\Processor\AfterOrder;

use Glopal\Connector\Api\CreateRequestDTOInterface;
use Glopal\Connector\Api\Data\OrderItemTotalCostInterface;
use Glopal\Connector\Api\Data\OrderTotalCostInterface;
use Glopal\Connector\Model\Order\Management;
use Magento\Sales\Api\Data\OrderInterface;

class TotalCosts implements AfterOrderCreateProcessorInterface
{
    /** @var Management */
    private $orderManagement;

    /**
     * @param Management $orderManagement
     */
    public function __construct(Management $orderManagement)
    {
        $this->orderManagement = $orderManagement;
    }

    /**
     * @inheritdoc
     */
    public function process(OrderInterface $order, CreateRequestDTOInterface $createRequestDTO): void
    {
        if (!$createRequestDTO->isTotalCostOrder()) {
            return;
        }

        $exchangeRate = $createRequestDTO->getExchangeRate();
        $summaryTotal = $createRequestDTO->getSummary()['total'];
        $summaryItems = $createRequestDTO->getSummary()['items'];
        $summaryDiscount = $createRequestDTO->getSummary()['discounts'];
        $summaryShipping = $createRequestDTO->getSummary()['shipping'];
        $summaryExtra = $createRequestDTO->getSummary()['extra'];

        $totalCostData = [
            OrderTotalCostInterface::BASE_TOTAL_DUTY_AMOUNT => $summaryTotal['duty_on_amount'] / $exchangeRate,
            OrderTotalCostInterface::TOTAL_DUTY_AMOUNT => $summaryTotal['duty_on_amount'],
            OrderTotalCostInterface::BASE_TOTAL_TAX_ON_DUTY => $summaryTotal['tax_on_duty'] / $exchangeRate,
            OrderTotalCostInterface::TOTAL_TAX_ON_DUTY => $summaryTotal['tax_on_duty'],
            OrderTotalCostInterface::BASE_ITEMS_DUTY_AMOUNT => $summaryItems['duty_on_amount'] / $exchangeRate,
            OrderTotalCostInterface::ITEMS_DUTY_AMOUNT => $summaryItems['duty_on_amount'],
            OrderTotalCostInterface::BASE_ITEMS_TAX_ON_DUTY => $summaryItems['tax_on_duty'] / $exchangeRate,
            OrderTotalCostInterface::ITEMS_TAX_ON_DUTY => $summaryItems['tax_on_duty'],
            OrderTotalCostInterface::BASE_DISCOUNT_DUTY_AMOUNT => $summaryDiscount['duty_on_amount'] / $exchangeRate,
            OrderTotalCostInterface::DISCOUNT_DUTY_AMOUNT => $summaryDiscount['duty_on_amount'],
            OrderTotalCostInterface::BASE_DISCOUNT_TAX_ON_DUTY => $summaryDiscount['tax_on_duty'] / $exchangeRate,
            OrderTotalCostInterface::DISCOUNT_TAX_ON_DUTY => $summaryDiscount['tax_on_duty'],
            OrderTotalCostInterface::BASE_SHIPPING_DUTY_AMOUNT => $summaryShipping['duty_on_amount'] / $exchangeRate,
            OrderTotalCostInterface::SHIPPING_DUTY_AMOUNT => $summaryShipping['duty_on_amount'],
            OrderTotalCostInterface::BASE_SHIPPING_TAX_ON_DUTY => $summaryShipping['tax_on_duty'] / $exchangeRate,
            OrderTotalCostInterface::SHIPPING_TAX_ON_DUTY => $summaryShipping['tax_on_duty'],
            OrderTotalCostInterface::BASE_EXTRA_DUTY_AMOUNT => $summaryExtra['duty_on_amount'] / $exchangeRate,
            OrderTotalCostInterface::EXTRA_DUTY_AMOUNT => $summaryExtra['duty_on_amount'],
            OrderTotalCostInterface::BASE_EXTRA_TAX_ON_DUTY => $summaryExtra['tax_on_duty'] / $exchangeRate,
            OrderTotalCostInterface::EXTRA_TAX_ON_DUTY => $summaryExtra['tax_on_duty']
        ];

        $totalCost = $this->orderManagement->addTotalCost($order, $totalCostData);

        $itemsData = $createRequestDTO->getItems();
        $orderTccId = $totalCost->getId();
        unset($totalCost);

        foreach ($order->getItems() as $orderItem) {
            if ($orderItem->getParentItemId()) {
                continue;
            }

            $orderItemSku = $orderItem->getSku();
            if (!isset($itemsData[$orderItemSku])) {
                continue;
            }

            $itemData = $itemsData[$orderItemSku];
            $totalItemCostData = [
                OrderItemTotalCostInterface::ORDER_TCC_ID => $orderTccId,
                OrderItemTotalCostInterface::BASE_DUTY_AMOUNT => $itemData['line_price']['duty_on_amount']
                    / $exchangeRate,
                OrderItemTotalCostInterface::DUTY_AMOUNT => $itemData['line_price']['duty_on_amount'],
                OrderItemTotalCostInterface::BASE_TAX_ON_DUTY => $itemData['line_price']['tax_on_duty']
                    / $exchangeRate,
                OrderItemTotalCostInterface::TAX_ON_DUTY => $itemData['line_price']['tax_on_duty'],
                OrderItemTotalCostInterface::DUTY_PERCENT => $itemData['duty_rate'],
            ];

            $this->orderManagement->addItemTotalCost($orderItem, $totalItemCostData);
        }

        unset($itemData);
        unset($itemsData);
    }
}
