<?php

namespace Glopal\Connector\Processor;

use Glopal\Connector\Helper\CartTransfer as CartTransferHelper;
use Glopal\Connector\Processor\CartTransfer\CartTransferProcessorInterface;
use Magento\Quote\Api\Data\CartInterface;
use UnexpectedValueException;

class CartTransfer
{
    /** @var CartTransferHelper */
    private $cartTransferHelper;

    /** @var CartTransferProcessorInterface[] */
    private $processorPool;

    /**
     * @param CartTransferHelper $cartTransferHelper
     * @param array $processorPool
     */
    public function __construct(CartTransferHelper $cartTransferHelper, array $processorPool = [])
    {
        $this->cartTransferHelper = $cartTransferHelper;
        $this->processorPool = $processorPool;
    }

    /**
     * Processing.
     *
     * @param CartInterface $quote
     * @return array
     */
    public function process(CartInterface $quote): array
    {
        $data = $quote->getData();

        foreach ($this->processorPool as $processor) {
            if (!$processor instanceof CartTransferProcessorInterface) {
                throw new UnexpectedValueException(
                    'Processor must implement ' . CartTransferProcessorInterface::class
                );
            }

            $processor->process($quote, $data);
        }

        $data = $this->cartTransferHelper->convertQuoteDataToArray($data);

        return $this->cartTransferHelper->quoteFilterRecursive($data);
    }
}
