<?php

namespace Glopal\Connector\Processor\CartTransfer;

use Glopal\Connector\Provider\CartTransfer\GiftMessageProvider;
use Magento\Quote\Api\Data\CartInterface;

class GiftMessages implements CartTransferProcessorInterface
{
    private const KEY_GIFT_MESSAGE = 'wi_gift_message_text';
    private const KEY_GIFT_MESSAGE_SENDER = 'wi_gift_message_from';
    private const KEY_GIFT_MESSAGE_RECIPIENT = 'wi_gift_message_to';

    /** @var GiftMessageProvider */
    private $giftMessageProvider;

    /**
     * @param GiftMessageProvider $giftMessageProvider
     */
    public function __construct(GiftMessageProvider $giftMessageProvider)
    {
        $this->giftMessageProvider = $giftMessageProvider;
    }

    /**
     * @inheritdoc
     */
    public function process(CartInterface $quote, array &$data): void
    {
        $quoteId = (int) $quote->getId();
        if (isset($data['gift_message_id'])) {
            $data[self::KEY_GIFT_MESSAGE] = $this->giftMessageProvider->getQuoteGiftMessage($quoteId);
            $data[self::KEY_GIFT_MESSAGE_SENDER] = $this->giftMessageProvider->getQuoteGiftMessageSender($quoteId);
            $data[self::KEY_GIFT_MESSAGE_RECIPIENT] = $this->giftMessageProvider->getQuoteGiftMessageRecipient(
                $quoteId
            );
        }

        foreach ($data['wi_items'] ?? [] as $itemKey => $item) {
            if (isset($item['gift_message_id'])) {
                $itemId = (int) $item['item_id'];
                $item[self::KEY_GIFT_MESSAGE] = $this->giftMessageProvider->getQuoteItemGiftMessage(
                    $quoteId,
                    $itemId
                );
                $item[self::KEY_GIFT_MESSAGE_SENDER] = $this->giftMessageProvider->getQuoteItemGiftMessageSender(
                    $quoteId,
                    $itemId
                );
                $item[self::KEY_GIFT_MESSAGE_RECIPIENT] = $this->giftMessageProvider->getQuoteItemGiftMessageRecipient(
                    $quoteId,
                    $itemId
                );

                $data['wi_items'][$itemKey] = $item;
            }
        }
    }
}
