<?php

namespace Glopal\Connector\Processor\Order;

use Glopal\Connector\Api\CreateRequestDTOInterface;
use Glopal\Connector\Helper\Order;
use Magento\Quote\Api\Data\CartInterface;

class Addresses implements OrderCreateProcessorInterface
{
    /** @var Order */
    private $orderHelper;

    /**
     * @param Order $orderHelper
     */
    public function __construct(Order $orderHelper)
    {
        $this->orderHelper = $orderHelper;
    }

    /**
     * @inheritdoc
     */
    public function process(CartInterface $quote, CreateRequestDTOInterface $createRequestDTO): void
    {
        $quote->getBillingAddress()->addData(
            $this->orderHelper->convertAddressFromRequest($createRequestDTO->getBillingAddress())
        );

        $customerShippingAddress = $this->orderHelper->convertAddressFromRequest(
            $createRequestDTO->getShippingAddress()
        );

        if ($createRequestDTO->getHubAddress()) {
            $hubAddress = $createRequestDTO->getHubAddress();
            // Adding a telephone number (from billing address) to hub address, required in Magento.
            $hubAddress['telephone'] = $createRequestDTO->getBillingAddress()['telephone'];

            // Hub address will be used for quote and order shipping address.
            // Customer origin shipping address will be saved as order attribute
            $quote->getShippingAddress()->addData($this->orderHelper->convertAddressFromRequest($hubAddress));
        } else {
            $quote->getShippingAddress()->addData($customerShippingAddress);
        }
    }
}
