<?php

namespace Glopal\Connector\Processor\Order;

use Glopal\Connector\Api\CreateRequestDTOInterface;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Quote\Model\Quote;

class Customer implements OrderCreateProcessorInterface
{
    /** @var CustomerRepositoryInterface */
    private $customerRepository;

    /**
     * @param CustomerRepositoryInterface $customerRepository
     */
    public function __construct(CustomerRepositoryInterface $customerRepository)
    {
        $this->customerRepository = $customerRepository;
    }

    /**
     * @inheritdoc
     */
    public function process(CartInterface $quote, CreateRequestDTOInterface $createRequestDTO): void
    {
        $quoteEmail = $createRequestDTO->getCustomerEmailAlias() ?? $createRequestDTO->getCustomerEmail();

        try {
            $customer = $this->customerRepository->get($createRequestDTO->getCustomerEmail());

            $quote->assignCustomer($customer);
            $quote->setCustomerIsGuest(false);
        } catch (NoSuchEntityException $e) {
            /** @var Quote $quote */
            $quote->setCustomerFirstname($createRequestDTO->getCustomerFirstName());
            $quote->setCustomerLastname($createRequestDTO->getCustomerLastName());

            $quote->setCustomerIsGuest(true);
        } finally {
            $quote->setCustomerEmail($quoteEmail);
        }
    }
}
