<?php

namespace Glopal\Connector\Processor\Order;

use Glopal\Connector\Api\CreateRequestDTOInterface;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Quote\Model\Quote\Address\RateFactory;
use Magento\Shipping\Model\Carrier\AbstractCarrier;
use Magento\Shipping\Model\CarrierFactory;

class Shipping implements OrderCreateProcessorInterface
{
    private const GL_SHIPPING_CODE = 'glopal';

    /** @var RateFactory */
    private $rateFactory;

    /** @var CarrierFactory */
    private $carrierFactory;

    /**
     * @param RateFactory $rateFactory
     * @param CarrierFactory $carrierFactory
     */
    public function __construct(RateFactory $rateFactory, CarrierFactory $carrierFactory)
    {
        $this->rateFactory = $rateFactory;
        $this->carrierFactory = $carrierFactory;
    }

    /**
     * @inheritdoc
     */
    public function process(CartInterface $quote, CreateRequestDTOInterface $createRequestDTO): void
    {
        $shippingData = $createRequestDTO->getShipping();
        $shippingAddress = $quote->getShippingAddress();

        $shippingMethodObject = null;
        $shippingMethodCode = $shippingData['service_code'] ?? null;
        if (null !== $shippingMethodCode) {
            $shippingMethodCode = explode('_', $shippingMethodCode)[0];

            /** @var bool|AbstractCarrier $shippingMethodObject */
            $shippingMethodObject = $this->carrierFactory->getIfActive($shippingMethodCode);
        }

        if ($shippingMethodObject) {
            $code = $shippingMethodObject->getCarrierCode();
            $shippingMethod = $code;
            $shippingMethodCode = $code . '_' . $code;
            $shippingMethodTitle = $shippingMethodObject->getConfigData('title');
        } else {
            $shippingMethod = $shippingMethodCode ?? self::GL_SHIPPING_CODE;
            $shippingMethodCode = !empty($shippingMethodCode)
                ? $shippingMethodCode . '_' . $shippingMethodCode
                : self::GL_SHIPPING_CODE . '_' . self::GL_SHIPPING_CODE;
            $shippingMethodTitle = $shippingData['display_name'] ?? $shippingData['service_method'];
        }

        $shippingAddress->setShippingMethod($shippingMethodCode);
        $shippingAddress->setShippingDescription($shippingMethodTitle);

        $shippingCharge = $createRequestDTO->getSummary()['shipping'];
        $rate = $this->rateFactory->create();
        $shippingAddress->addShippingRate(
            $rate->setCode($shippingMethodCode)
                ->setMethod($shippingMethod)
                ->setMethodTitle($shippingMethodTitle)
                ->setMethodDescription($shippingMethodTitle)
                ->setCarrier($shippingMethod)
                ->setCarrierTitle($shippingMethodTitle)
                ->setPrice($shippingCharge['total'])
        );
    }
}
