<?php

namespace Glopal\Connector\Provider\CartTransfer;

use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Helper\Utils;
use Magento\Framework\UrlInterface;
use Magento\Quote\Api\Data\CartInterface;

class CallBackUrlsProvider
{
    /** @var Configuration */
    private $configuration;

    /** @var UrlInterface */
    private $urlBuilder;

    /** @var Utils */
    private $utils;

    /**
     * @param Configuration $configuration
     * @param UrlInterface $urlBuilder
     * @param Utils $utils
     */
    public function __construct(
        Configuration $configuration,
        UrlInterface $urlBuilder,
        Utils $utils
    ) {
        $this->configuration = $configuration;
        $this->urlBuilder = $urlBuilder;
        $this->utils = $utils;
    }

    /**
     * Retrieve callback inventory check URL.
     *
     * @param null|string|int $storeId
     *
     * @return null|string
     */
    public function getInventoryCheckUrl($storeId = null): ?string
    {
        return $this->configuration->getCartValue('callback_url/inventory_check', $storeId);
    }

    /**
     * Retrieve callback checkout purchase URL.
     *
     * @param CartInterface $quote
     * @param null $storeId
     *
     * @return null|string
     */
    public function getCheckoutPurchaseUrl(CartInterface $quote, $storeId = null): ?string
    {
        $callbackUrl = $this->configuration->getCartValue('callback_url/checkout_purchase', $storeId);

        if (empty($callbackUrl)) {
            $cartId = $quote->getId();

            if (empty($cartId)) {
                return null;
            }

            $secretKey = $this->configuration->getGeneralValue('key');
            $callbackUrl = $this->urlBuilder->getUrl('glopal_connector/checkout/callbackPurchase')
                . '?hmac='
                . $this->utils->buildHmacKey($cartId, $secretKey);
        }

        return $callbackUrl;
    }

    /**
     * Retrieve callback checkout success URL.
     *
     * @param string $returnTo
     * @return string
     */
    public function getCustomSuccessPageUrl(string $returnTo): string
    {
        return $this->urlBuilder->getUrl(
            'glopal_connector/checkout/success',
            ['_query' => ['returnTo' => $returnTo]]
        );
    }
}
