<?php

namespace Glopal\Connector\Provider\CartTransfer;

use Glopal\Connector\Helper\Configuration;
use Magento\Catalog\Model\Product\Image\UrlBuilder;

class SmallImageProvider
{
    /** @var Configuration */
    private $configuration;

    /** @var UrlBuilder */
    private $productImageUrlBuilder;

    /**
     * @param Configuration $configuration
     * @param UrlBuilder $productImageUrlBuilder
     */
    public function __construct(Configuration $configuration, UrlBuilder $productImageUrlBuilder)
    {
        $this->configuration = $configuration;
        $this->productImageUrlBuilder = $productImageUrlBuilder;
    }

    /**
     * Retrieve full image path for 'small_image' (include caching).
     *
     * @param string $path
     *
     * @return null|string
     */
    public function get(string $path): ?string
    {
        if (empty($path)) {
            return null;
        }

        $imageDisplayArea = $this->configuration->getGeneralValue('image_display_area');
        if (empty($imageDisplayArea)) {
            return null;
        }

        return $this->productImageUrlBuilder->getUrl($path, $imageDisplayArea);
    }
}
