<?php

namespace Glopal\Connector\Provider\CartTransfer;

use Glopal\Connector\Helper\Configuration;
use Magento\Catalog\Api\Data\ProductInterface;

class SpecificProductAttributesProvider
{
    /** @var Configuration */
    private $configuration;

    /**
     * @param Configuration $configuration
     */
    public function __construct(Configuration $configuration)
    {
        $this->configuration = $configuration;
    }

    /**
     * Retrieve custom product attributes based on configuration.
     *
     * @param ProductInterface $product
     * @param null|int|string $storeId
     *
     * @return array
     */
    public function get(ProductInterface $product, $storeId): array
    {
        $result = [];
        $attributeSetId = $product->getAttributeSetId();
        $attributeMapping = $this->configuration->getCartProductDataValue('extra_keys_mapping', $storeId);
        $attributeNames = $this->configuration->getCartProductDataValue('extra_keys_names', $storeId);
        $glopalKeys = array_column($attributeNames, 'code');

        foreach ($attributeMapping as $item) {
            if (!in_array($item['glopal_key'], $glopalKeys)) {
                continue;
            }

            if ($item['product_attribute_set'] == $attributeSetId) {
                $result[$item['glopal_key']] = $product->getData($item['product_attribute']);
            }
        }

        return $result;
    }
}
