<?php

namespace Glopal\Connector\Provider;

use Glopal\Connector\Model\Price\Calculator;
use Glopal\Connector\Model\Price\CalculatorFactory;
use Magento\Tax\Model\Config;

class PriceCalculatorProvider
{
    /** @var CalculatorFactory */
    private $priceCalculatorFactory;

    /** @var Config */
    private $taxConfig;

    /**
     * @param CalculatorFactory $priceCalculatorFactory
     * @param Config $taxConfig
     */
    public function __construct(CalculatorFactory $priceCalculatorFactory, Config $taxConfig)
    {
        $this->priceCalculatorFactory = $priceCalculatorFactory;
        $this->taxConfig = $taxConfig;
    }

    /**
     * Gets price data from configuration.
     *
     * @param null|string|int $storeId
     * @return Calculator
     */
    public function get($storeId = null): Calculator
    {
        return $this->priceCalculatorFactory->create([
            'priceIncludeTax' => (bool)$this->taxConfig->priceIncludesTax($storeId),
            'applyTaxAfterDiscount' => $this->taxConfig->applyTaxAfterDiscount($storeId)
        ]);
    }
}
