<?php

namespace Glopal\Connector\Provider;

use Magento\Catalog\Model\ResourceModel\Product;
use Magento\Eav\Model\ResourceModel\Entity\Attribute\Set\CollectionFactory;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Adapter\AdapterInterface;

class ProductAttributeSetsProvider
{
    /** @var AdapterInterface */
    private $connection;

    /** @var CollectionFactory */
    private $collectionFactory;

    /** @var Product */
    private $product;

    /**
     * @param ResourceConnection $resource
     * @param CollectionFactory $collectionFactory
     * @param Product $product
     */
    public function __construct(
        ResourceConnection $resource,
        CollectionFactory $collectionFactory,
        Product $product
    ) {
        $this->connection = $resource->getConnection();
        $this->collectionFactory = $collectionFactory;
        $this->product = $product;
    }

    /**
     * Gets options.
     *
     * @return array
     */
    public function getOptions(): array
    {
        return $this->collectionFactory->create()
            ->setEntityTypeFilter($this->product->getTypeId())
            ->toOptionArray();
    }

    /**
     * Get attribute set items.
     *
     * @return array
     */
    public function getAttributeSetsItems(): array
    {
        $select = $this->connection->select()
            ->from(['eas' => 'eav_attribute_set'], [])
            ->joinLeft(
                ['eea' => 'eav_entity_attribute'],
                'eas.attribute_set_id = eea.attribute_set_id',
                ['attribute_set_id', 'attribute_id']
            )->where('eas.entity_type_id = ?', $this->product->getTypeId());

        $queryResult = $this->connection->fetchAll($select);
        $result = [];

        foreach ($queryResult as $item) {
            $result[$item['attribute_id']][] = $item['attribute_set_id'];
        }

        return $result;
    }
}
