<?php

namespace Glopal\Connector\Provider;

use Magento\Eav\Model\AttributeRepository;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Exception\InputException;

class ProductAttributesOptionsProvider
{
    /** @var AttributeRepository */
    private $attributeRepository;

    /** @var SearchCriteriaBuilder */
    private $searchCriteriaBuilder;

    /** @var ProductAttributeSetsProvider */
    private $productAttributeSetsProvider;

    /**
     * @param AttributeRepository $attributeRepository
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param ProductAttributeSetsProvider $productAttributeSetsProvider
     */
    public function __construct(
        AttributeRepository $attributeRepository,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        ProductAttributeSetsProvider $productAttributeSetsProvider
    ) {
        $this->attributeRepository = $attributeRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->productAttributeSetsProvider = $productAttributeSetsProvider;
    }

    /**
     * Gets options.
     *
     * @return array
     * @throws InputException
     */
    public function getOptions(): array
    {
        $options = [];

        $attributesList = $this->attributeRepository->getList(
            'catalog_product',
            $this->searchCriteriaBuilder->create()
        );

        $attributeSetsItems = $this->productAttributeSetsProvider->getAttributeSetsItems();

        foreach ($attributesList->getItems() as $attribute) {
            $attributeSetIds = [];
            if (isset($attributeSetsItems[$attribute->getAttributeId()])) {
                $attributeSetIds = $attributeSetsItems[$attribute->getAttributeId()];
            }

            $options[] = [
                'label' => $attribute->getDefaultFrontendLabel(),
                'value' => $attribute->getAttributeCode(),
                'params' => [
                    'data-attribute-set-id' => implode(',', $attributeSetIds)
                ]
            ];
        }

        return $options;
    }
}
