<?php

namespace Glopal\Connector\Response;

use Glopal\Connector\Helper\ModuleData;
use Glopal\Connector\Processor\CartTransfer;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Serialize\Serializer\Base64Json;
use Magento\Quote\Api\Data\CartInterface;

class CartResponseBuilder
{
    /** @var CartTransfer */
    private $cartTransferProcessor;

    /** @var ModuleData */
    private $dataHelper;

    /** @var Base64Json */
    private $base64JsonSerializer;

    /** @var RequestInterface */
    private $request;

    /**
     * @param CartTransfer $cartTransferProcessor
     * @param ModuleData $dataHelper
     * @param Base64Json $base64JsonSerializer
     * @param RequestInterface $request
     */
    public function __construct(
        CartTransfer $cartTransferProcessor,
        ModuleData $dataHelper,
        Base64Json $base64JsonSerializer,
        RequestInterface $request
    ) {
        $this->dataHelper = $dataHelper;
        $this->cartTransferProcessor = $cartTransferProcessor;
        $this->base64JsonSerializer = $base64JsonSerializer;
        $this->request = $request;
    }

    /**
     * Build json cart transfer (quote) response.
     *
     * @param CartInterface $quote
     * @param bool $encode
     *
     * @return array
     */
    public function build(CartInterface $quote, bool $encode = true): array
    {
        $moduleVersion = $this->dataHelper->getExtensionVersion();
        $response = [
            'magento_version' => $this->dataHelper->getMagentoVersion(),
            'connector_version' => $moduleVersion,
        ];

        $quoteData = $this->cartTransferProcessor->process($quote);

        if ($encode) {
            $encodedQuoteData = $this->base64JsonSerializer->serialize($quoteData);
            $key = $this->dataHelper->getGlopalKey();
            $response['quote'] = $encodedQuoteData;
            $response['hash'] = hash_hmac('sha256', $encodedQuoteData, $key);
        } else {
            $response['quote'] = $quoteData;
        }

        if ($this->isDebugAvailable()) {
            $response['quote_debug'] = $quoteData;
        }

        return $response;
    }

    /**
     * Check is debug available.
     *
     * @return bool
     */
    private function isDebugAvailable(): bool
    {
        return (bool)$this->request->getParam('debug');
    }
}
