<?php

namespace Glopal\Connector\Service;

use Exception;
use Glopal\Connector\Provider\CartTransfer\RequestUrlProvider;
use Magento\Framework\HTTP\Client\Curl;
use Magento\Framework\HTTP\Client\CurlFactory;
use Magento\Framework\Serialize\SerializerInterface;
use Psr\Log\LoggerInterface;

class GlopalCheckout
{
    /** @var RequestUrlProvider */
    private $requestUrlProvider;

    /** @var CurlFactory */
    private $curlClientFactory;

    /** @var SerializerInterface */
    private $serializer;

    /** @var LoggerInterface */
    private $logger;

    /**
     * @param RequestUrlProvider $requestUrlProvider
     * @param CurlFactory $curlClientFactory
     * @param SerializerInterface $serializer
     * @param LoggerInterface $logger
     */
    public function __construct(
        RequestUrlProvider $requestUrlProvider,
        CurlFactory $curlClientFactory,
        SerializerInterface $serializer,
        LoggerInterface $logger
    ) {
        $this->requestUrlProvider = $requestUrlProvider;
        $this->curlClientFactory = $curlClientFactory;
        $this->serializer = $serializer;
        $this->logger = $logger;
    }

    /**
     * Request Glopal checkout url.
     *
     * @param string $requestBody
     * @return string|null
     * @throws Exception
     */
    public function requestCartTransfer(string $requestBody): ?string
    {
        $requestUrl = $this->requestUrlProvider->get();

        if (null === $requestUrl) {
            return null;
        }

        /** @var Curl $client */
        $client = $this->curlClientFactory->create();
        $client->addHeader('Content-Type', 'application/json');
        $client->addHeader('Accept', 'application/json');
        $client->post($requestUrl, $requestBody);

        if ($client->getStatus() !== 200) {
            return null;
        }

        $responseBody = $client->getBody();
        $response = $this->serializer->unserialize($responseBody);

        if ($response['status'] !== 'ok') {
            $this->logger->error(
                'Bad response from cart transfer',
                [
                    'response_code' => $response['code'] ?? '',
                    'response_context' => $response['context'] ?? ''
                ]
            );

            return null;
        }

        return $response['cart_url'] ?? null;
    }
}
