<?php

namespace Glopal\Connector\Setup\Patch\Data;

use Magento\Config\Model\ResourceModel\Config;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\Patch\DataPatchInterface;
use Magento\Framework\Setup\Patch\PatchRevertableInterface;

class AddPaymentMethodConfiguration implements DataPatchInterface, PatchRevertableInterface
{
    /** @var ModuleDataSetupInterface */
    private $moduleDataSetup;

    /** @var Config */
    private $resourceConfig;

    /** @var array */
    private $glopalPaymentConfigValuesMap = [
        'payment/glopal/active' => 1,
        'payment/glopal/title' => 'Glopal',
    ];

    /**
     * @param ModuleDataSetupInterface $moduleDataSetup
     * @param Config $resourceConfig
     */
    public function __construct(ModuleDataSetupInterface $moduleDataSetup, Config $resourceConfig)
    {
        $this->moduleDataSetup = $moduleDataSetup;
        $this->resourceConfig = $resourceConfig;
    }

    /**
     * @inheritdoc
     */
    public function apply()
    {
        $connection = $this->moduleDataSetup->getConnection();
        $connection->startSetup();

        foreach ($this->glopalPaymentConfigValuesMap as $path => $value) {
            $this->resourceConfig->saveConfig($path, $value);
        }

        $connection->endSetup();

        return $this;
    }

    /**
     * @inheritdoc
     *
     * @SuppressWarnings(PHPMD.UnusedLocalVariable)
     */
    public function revert()
    {
        foreach ($this->glopalPaymentConfigValuesMap as $path => $value) {
            $this->resourceConfig->deleteConfig($path);
        }
    }

    /**
     * @inheritdoc
     */
    public static function getDependencies()
    {
        return [];
    }

    /**
     * @inheritdoc
     */
    public function getAliases()
    {
        return [];
    }
}
