<?php

namespace Glopal\Connector\Test\Unit;

use Glopal\Connector\Helper\CartTransfer as CartTransferHelper;
use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Processor\CartTransfer;
use Glopal\Connector\Processor\CartTransfer\Checkout;
use Glopal\Connector\Processor\CartTransfer\GiftMessages;
use Glopal\Connector\Processor\CartTransfer\QuoteItems;
use Glopal\Connector\Processor\CartTransfer\TaxOptions;
use Glopal\Connector\Provider\CartTransfer\CallBackUrlsProvider;
use Glopal\Connector\Provider\CartTransfer\CombineProductDataProvider;
use Glopal\Connector\Provider\CartTransfer\GiftMessageProvider;
use Glopal\Connector\Provider\CartTransfer\OrderOptionsProvider;
use Glopal\Connector\Provider\CartTransfer\SmallImageProvider;
use Glopal\Connector\Provider\CartTransfer\SpecificProductAttributesProvider;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Catalog\Model\Product\Type\AbstractType;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\DataObject;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Quote\Api\Data\CartItemInterface;
use Magento\Quote\Model\Quote\Item\Option;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;

// phpcs:ignoreFile
class CartTransferTestHelper extends TestCase
{
    private $storeId = 1;
    private $quoteId = 123;

    public function getCartTransferProcessor(MockObject $quote)
    {
        $quote->method('getStoreId')
            ->willReturn($this->storeId);

        $quote->method('getId')
            ->willReturn($this->quoteId);

        // Checkout processor
        $inventoryCheckUrl = 'https://example.com/inventory-check';
        $checkoutPurchaseUrl = 'https://example.com/checkout-purchase';

        $callBackUrlsProvider = $this->createMock(CallBackUrlsProvider::class);
        $callBackUrlsProvider->method('getInventoryCheckUrl')
            ->with($this->storeId)
            ->willReturn($inventoryCheckUrl);
        $callBackUrlsProvider->method('getCheckoutPurchaseUrl')
            ->with($quote, $this->storeId)
            ->willReturn($checkoutPurchaseUrl);

        $checkoutProcessor = new Checkout($callBackUrlsProvider);

        // Gift Messages processor
        $giftMessageProvider = $this->createMock(GiftMessageProvider::class);
        $giftMessageProvider->method('getQuoteGiftMessage')
            ->with($this->quoteId)
            ->willReturn('Quote gift message text');

        $giftMessagesProcessor = new GiftMessages($giftMessageProvider);

        // Quote Items processor
        $productRepositoryMock = $this->getMockForAbstractClass(
            ProductRepositoryInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getById']
        );
        $requestMock = $this->getMockForAbstractClass(
            RequestInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getParam']
        );
        $combineProductDataProviderMock = $this->createMock(CombineProductDataProvider::class);
        $smallImageProviderMock = $this->createMock(SmallImageProvider::class);
        $specificProductAttributesProviderMock = $this->createMock(SpecificProductAttributesProvider::class);
        $orderOptionsProviderMock = $this->createMock(OrderOptionsProvider::class);
        $cartItemInterfaceMock = $this->getMockForAbstractClass(
            CartItemInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getProductId', 'getData', 'getProduct', 'getOptionByCode', 'getBuyRequest']
        );
        $quote->method('getAllVisibleItems')
            ->willReturn([$cartItemInterfaceMock]);
        $modelProductMock = $this->getMockForAbstractClass(
            ProductInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getData']
        );
        $itemProductMock = $this->getMockForAbstractClass(
            ProductInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getTypeInstance', 'getData']
        );
        $simpleProductMock = $this->getMockForAbstractClass(ProductInterface::class);
        $optionMock = $this->createMock(Option::class);
        $buyRequestInstanceMock = $this->getMockBuilder(DataObject::class)
            ->disableOriginalConstructor()
            ->setMethods(['getData'])
            ->getMock();

        $cartItemInterfaceMock->method('getProductId')
            ->willReturn(10);

        $cartItemInterfaceMock->method('getData')
            ->willReturn([
                'item_id' => '5',
                'quote_id' => '3',
                'created_at' => '2023-03-16 16:23:37',
                'updated_at' => '2023-03-16 16:23:37',
                'product_id' => '1396',
                'store_id' => 1,
            ]);

        $cartItemInterfaceMock->method('getProduct')
            ->willReturn($itemProductMock);

        $cartItemInterfaceMock->method('getOptionByCode')
            ->with($this->equalTo('simple_product'))
            ->willReturn($optionMock);

        $productRepositoryMock->method('getById')
            ->with(10, false, $this->storeId)
            ->willReturn($modelProductMock);

        $combineProductDataProviderMock->method('get')
            ->with($modelProductMock, $itemProductMock)
            ->willReturn([
                'name' => 'Product',
                'description' => 'This is the product',
                'price' => 15.00,
                'small_image' => 'image.jpg',
            ]);

        $smallImageProviderMock->method('get')
            ->with('image.jpg')
            ->willReturn('https://example.com/media/catalog/product/w/j/image.jpg');

        $specificProductAttributesProviderMock->method('get')
            ->with($modelProductMock, $this->storeId)
            ->willReturn(['key' => 'some_value']);

        $requestMock->method('getParam')
            ->with('debug')
            ->willReturn(true);

        $optionMock->method('getProduct')
            ->willReturn($simpleProductMock);

        $itemProductTypeInstanceMock = $this->createMock(AbstractType::class);

        $itemProductMock->method('getTypeInstance')
            ->with($this->equalTo(true))
            ->willReturn($itemProductTypeInstanceMock);

        $itemProductTypeInstanceMock->method('getOrderOptions')
            ->with($itemProductMock)
            ->willReturn(['order' => 'options']);

        $orderOptionsProviderMock->method('get')
            ->with(['order' => 'options'], $simpleProductMock)
            ->willReturn([
                'attributes_info' => [
                    [
                        'label' => 'Size',
                        'value' => 'S',
                        'option_id' => '44',
                        'option_value' => '166',
                    ],
                ],
                'simple_name' => 'Product name',
                'simple_sku' => 'SKU',
            ]);

        $cartItemInterfaceMock->method('getBuyRequest')
            ->willReturn($buyRequestInstanceMock);

        $buyRequestInstanceMock->method('getData')
            ->willReturn([
                'product' => '1396',
                'super_attribute' => [
                    '144' => '166',
                    '93' => '49'
                ],
                'qty' => 1,
                'original_qty' => 1
            ]);

        $quoteItemsProcessor = new QuoteItems(
            $productRepositoryMock,
            $requestMock,
            $combineProductDataProviderMock,
            $smallImageProviderMock,
            $specificProductAttributesProviderMock,
            $orderOptionsProviderMock
        );

        // Tax Options processor
        $configuration = $this->createMock(Configuration::class);

        $configPathPool = [
            'price_includes_tax' => 'tax/calculation/price_includes_tax',
            'algorithm' => 'tax/calculation/algorithm',
            'apply_after_discount' => 'tax/calculation/apply_after_discount',
            'discount_tax' => 'tax/calculation/discount_tax',
            'apply_tax_on' => 'tax/calculation/apply_tax_on',
            'cross_border_trade_enabled' => 'tax/calculation/cross_border_trade_enabled',
            'tax_display_shipping' => 'tax/display/shipping',
            'tax_display_type' => 'tax/display/type',
            'tax_calculation_shipping_includes_tax' => 'tax/calculation/shipping_includes_tax',
        ];

        $configuration->method('getValue')
            ->willReturnMap([
                ['tax/calculation/price_includes_tax', $this->storeId, '0'],
                ['tax/calculation/algorithm', $this->storeId, 'TOTAL_BASE_CALCULATION'],
                ['tax/calculation/apply_after_discount', $this->storeId, '1'],
                ['tax/calculation/discount_tax', $this->storeId, '0'],
                ['tax/calculation/apply_tax_on', $this->storeId, '0'],
                ['tax/calculation/cross_border_trade_enabled', $this->storeId, ''],
                ['tax/display/shipping', $this->storeId, '1'],
                ['tax/display/type', $this->storeId, '1'],
                ['tax/calculation/shipping_includes_tax', $this->storeId, '0'],
            ]);

        $taxOptionsProcessor = new TaxOptions($configuration, $configPathPool);

        $processorPool = [
            'quote_items' => $quoteItemsProcessor,
            'tax_options' => $taxOptionsProcessor,
            'checkout' => $checkoutProcessor,
            "gift_messages" => $giftMessagesProcessor,
        ];

        $quoteData = [
            'entity_id' => '3',
            'store_id' => 1,
            'created_at' => '2023-03-16 16:23:37',
            'updated_at' => '2023-03-16 16:23:37',
            'gift_message_id' => 1,
        ];

        $quote->method('getData')
            ->willReturn($quoteData);

        $cartTransferHelper = new CartTransferHelper(new Json());

        return new CartTransfer($cartTransferHelper, $processorPool);
    }

    public function getCartTransferProcessorForEmptyQuote(MockObject $quote)
    {
        $quote->method('getStoreId')
            ->willReturn($this->storeId);

        $quote->method('getId')
            ->willReturn(null);

        $quote->method('getAllVisibleItems')
            ->willReturn([]);

        // Checkout processor
        $inventoryCheckUrl = 'https://example.com/inventory-check';
        $checkoutPurchaseUrl = 'https://example.com/checkout-purchase';

        $callBackUrlsProvider = $this->createMock(CallBackUrlsProvider::class);
        $callBackUrlsProvider->method('getInventoryCheckUrl')
            ->with($this->storeId)
            ->willReturn($inventoryCheckUrl);
        $callBackUrlsProvider->method('getCheckoutPurchaseUrl')
            ->with($quote, $this->storeId)
            ->willReturn($checkoutPurchaseUrl);

        $checkoutProcessor = new Checkout($callBackUrlsProvider);

        // Gift Messages processor
        $giftMessageProvider = $this->createMock(GiftMessageProvider::class);
        $giftMessageProvider->method('getQuoteGiftMessage')
            ->with(null)
            ->willReturn('Quote gift message text');

        $giftMessagesProcessor = new GiftMessages($giftMessageProvider);

        // Quote Items processor
        $productRepositoryMock = $this->getMockForAbstractClass(
            ProductRepositoryInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getById']
        );
        $requestMock = $this->getMockForAbstractClass(
            RequestInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getParam']
        );
        $combineProductDataProviderMock = $this->createMock(CombineProductDataProvider::class);
        $smallImageProviderMock = $this->createMock(SmallImageProvider::class);
        $specificProductAttributesProviderMock = $this->createMock(SpecificProductAttributesProvider::class);
        $orderOptionsProviderMock = $this->createMock(OrderOptionsProvider::class);

        $quoteItemsProcessor = new QuoteItems(
            $productRepositoryMock,
            $requestMock,
            $combineProductDataProviderMock,
            $smallImageProviderMock,
            $specificProductAttributesProviderMock,
            $orderOptionsProviderMock
        );

        // Tax Options processor
        $configuration = $this->createMock(Configuration::class);

        $configPathPool = [
            'price_includes_tax' => 'tax/calculation/price_includes_tax',
            'algorithm' => 'tax/calculation/algorithm',
            'apply_after_discount' => 'tax/calculation/apply_after_discount',
            'discount_tax' => 'tax/calculation/discount_tax',
            'apply_tax_on' => 'tax/calculation/apply_tax_on',
            'cross_border_trade_enabled' => 'tax/calculation/cross_border_trade_enabled',
            'tax_display_shipping' => 'tax/display/shipping',
            'tax_display_type' => 'tax/display/type',
            'tax_calculation_shipping_includes_tax' => 'tax/calculation/shipping_includes_tax',
        ];

        $configuration->method('getValue')
            ->willReturnMap([
                ['tax/calculation/price_includes_tax', $this->storeId, '0'],
                ['tax/calculation/algorithm', $this->storeId, 'TOTAL_BASE_CALCULATION'],
                ['tax/calculation/apply_after_discount', $this->storeId, '1'],
                ['tax/calculation/discount_tax', $this->storeId, '0'],
                ['tax/calculation/apply_tax_on', $this->storeId, '0'],
                ['tax/calculation/cross_border_trade_enabled', $this->storeId, ''],
                ['tax/display/shipping', $this->storeId, '1'],
                ['tax/display/type', $this->storeId, '1'],
                ['tax/calculation/shipping_includes_tax', $this->storeId, '0'],
            ]);

        $taxOptionsProcessor = new TaxOptions($configuration, $configPathPool);

        $processorPool = [
            'quote_items' => $quoteItemsProcessor,
            'tax_options' => $taxOptionsProcessor,
            'checkout' => $checkoutProcessor,
            "gift_messages" => $giftMessagesProcessor,
        ];

        $quoteData = [];

        $quote->method('getData')
            ->willReturn($quoteData);

        $cartTransferHelper = new CartTransferHelper(new Json());

        return new CartTransfer($cartTransferHelper, $processorPool);
    }

    public function getExpectedResult()
    {
        return [
            'entity_id' => '3',
            'store_id' => 1,
            'created_at' => '2023-03-16 16:23:37',
            'updated_at' => '2023-03-16 16:23:37',
            'gift_message_id' => 1,
            'wi_gift_message_text' => 'Quote gift message text',
            'wi_gift_message_from' => null,
            'wi_gift_message_to' => null,
            'wi_items' => [
                [
                    "item_id" => "5",
                    "quote_id" => "3",
                    "product_id" => "1396",
                    "store_id" => 1,
                    "wi_product" => [
                        'name' => 'Product',
                        'description' => 'This is the product',
                        'price' => 15,
                        'small_image' => 'https://example.com/media/catalog/product/w/j/image.jpg',
                        'key' => 'some_value',
                    ],
                    'created_at' => '2023-03-16 16:23:37',
                    'updated_at' => '2023-03-16 16:23:37',
                    'product' => [],
                    'wi_product_model' => null,
                    'wi_product_item' => null,
                    'wi_order_options' => [
                        'attributes_info' => [
                            [
                                'label' => 'Size',
                                'value' => 'S',
                                'option_id' => '44',
                                'option_value' => '166',
                            ]
                        ],
                        'simple_name' => 'Product name',
                        'simple_sku' => 'SKU',
                    ],
                    'wi_buy_request' => [
                        'product' => '1396',
                        'super_attribute' => [
                            144 => '166',
                            93 => '49',
                        ],
                        'qty' => 1,
                        'original_qty' => 1,
                    ],
                ]
            ],
            'wi_tax_options' => [
                'price_includes_tax' => '0',
                'algorithm' => 'TOTAL_BASE_CALCULATION',
                'apply_after_discount' => '1',
                'discount_tax' => '0',
                'apply_tax_on' => '0',
                'cross_border_trade_enabled' => '',
                'tax_display_shipping' => '1',
                'tax_display_type' => '1',
                'tax_calculation_shipping_includes_tax' => '0'
            ],
            'wi_checkout' => [
                'inventory_check' => 'https://example.com/inventory-check',
                'checkout_purchase' => 'https://example.com/checkout-purchase'
            ],
        ];
    }

    public function getExpectedResultForEmptyQuote()
    {
        return [
            'wi_items' => [
            ],
            'wi_tax_options' => [
                'price_includes_tax' => '0',
                'algorithm' => 'TOTAL_BASE_CALCULATION',
                'apply_after_discount' => '1',
                'discount_tax' => '0',
                'apply_tax_on' => '0',
                'cross_border_trade_enabled' => '',
                'tax_display_shipping' => '1',
                'tax_display_type' => '1',
                'tax_calculation_shipping_includes_tax' => '0'
            ],
            'wi_checkout' => [
                'inventory_check' => 'https://example.com/inventory-check',
                'checkout_purchase' => 'https://example.com/checkout-purchase'
            ],
        ];
    }
}
