<?php

namespace Glopal\Connector\Test\Unit\Helper;

use Glopal\Connector\Helper\CartTransfer;
use Magento\Framework\Serialize\SerializerInterface;
use PHPUnit\Framework\TestCase;

// phpcs:ignoreFile
class CartTransferTest extends TestCase
{
    private $cartTransfer;
    private $serializer;

    protected function setUp(): void
    {
        $this->serializer = $this->getMockForAbstractClass(SerializerInterface::class);
        $this->cartTransfer = new CartTransfer($this->serializer);
    }

    public function testConvertQuoteDataToArray()
    {
        $quoteData = ['key' => 'value', 'object' => new \stdClass()];

        $this->serializer
            ->expects($this->once())
            ->method('serialize')
            ->with($quoteData)
            ->willReturn('serialized');

        $this->serializer
            ->expects($this->once())
            ->method('unserialize')
            ->with('serialized')
            ->willReturn(['key' => 'value']);

        $result = $this->cartTransfer->convertQuoteDataToArray($quoteData);
        $this->assertEquals(['key' => 'value'], $result);
    }

    public function testQuoteFilterRecursiveWithUenc()
    {
        $data = [
            'uenc' => 'test',
            'info_buyRequest' => ['test' => 'test'],
            '_cache' => ['test' => 'test'],
            'test' => [
                'sub_test' => 'test',
                '_cache' => 'test'
            ]
        ];
        $this->assertEquals(['test' => ['sub_test' => 'test']], $this->cartTransfer->quoteFilterRecursive($data));
    }
}
