<?php

namespace Glopal\Connector\Test\Unit\Processor\CartTransfer;

use Glopal\Connector\Processor\CartTransfer\Checkout;
use Glopal\Connector\Provider\CartTransfer\CallBackUrlsProvider;
use Magento\Quote\Api\Data\CartInterface;
use PHPUnit\Framework\TestCase;

// phpcs:ignoreFile
class CheckoutTest extends TestCase
{
    public function testProcess()
    {
        $storeId = 1;
        $inventoryCheckUrl = 'https://example.com/inventory-check';
        $checkoutPurchaseUrl = 'https://example.com/checkout-purchase';

        $cart = $this->getMockForAbstractClass(
            CartInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getStoreId']
        );
        $cart->expects($this->once())->method('getStoreId')->willReturn($storeId);

        $callBackUrlsProvider = $this->createMock(CallBackUrlsProvider::class);
        $callBackUrlsProvider->expects($this->once())
            ->method('getInventoryCheckUrl')
            ->with($storeId)
            ->willReturn($inventoryCheckUrl);
        $callBackUrlsProvider->expects($this->once())
            ->method('getCheckoutPurchaseUrl')
            ->with($cart, $storeId)
            ->willReturn($checkoutPurchaseUrl);

        $checkout = new Checkout($callBackUrlsProvider);

        $result = [
            'wi_checkout' => [
                'inventory_check' => $inventoryCheckUrl,
                'checkout_purchase' => $checkoutPurchaseUrl
            ]
        ];

        $checkout->process($cart, $result);
    }
}
