<?php

namespace Glopal\Connector\Test\Unit\Processor\CartTransfer;

use Glopal\Connector\Processor\CartTransfer\GiftMessages;
use Glopal\Connector\Provider\CartTransfer\GiftMessageProvider;
use Magento\Quote\Api\Data\CartInterface;
use PHPUnit\Framework\TestCase;

// phpcs:ignoreFile
class GiftMessagesTest extends TestCase
{
    private $cart;
    private $giftMessageProvider;
    private $quoteId = 123;

    public function setUp(): void
    {
        $this->cart = $this->getMockForAbstractClass(
            CartInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getId']
        );
        $this->giftMessageProvider = $this->createMock(GiftMessageProvider::class);
    }

    public function testProcessWithQuoteMessage()
    {
        $data = [
            'gift_message_id' => 1,
            'wi_items' => [
                0 => [
                    'gift_message_id' => 2,
                    'item_id' => 222
                ]
            ]
        ];

        $this->giftMessageProvider->expects($this->once())
            ->method('getQuoteGiftMessage')
            ->with($this->quoteId)
            ->willReturn('Quote gift message text');
        $this->giftMessageProvider->expects($this->once())
            ->method('getQuoteGiftMessageSender')
            ->with($this->quoteId)
            ->willReturn('Sender');
        $this->giftMessageProvider->expects($this->once())
            ->method('getQuoteGiftMessageRecipient')
            ->with($this->quoteId)
            ->willReturn('Recipient');

        $this->cart->expects($this->any())->method('getId')->willReturn($this->quoteId);

        $this->giftMessageProvider->expects($this->once())
            ->method('getQuoteItemGiftMessage')
            ->with($this->quoteId, 222)
            ->willReturn('Quote item 222 gift message text');
        $this->giftMessageProvider->expects($this->once())
            ->method('getQuoteItemGiftMessageSender')
            ->with($this->quoteId, 222)
            ->willReturn('Quote item sender');
        $this->giftMessageProvider->expects($this->once())
            ->method('getQuoteItemGiftMessageRecipient')
            ->with($this->quoteId, 222)
            ->willReturn('Quote item recipient');

        $giftMessages = new GiftMessages($this->giftMessageProvider);
        $giftMessages->process($this->cart, $data);

        $expected = [
            'gift_message_id' => 1,
            'wi_gift_message_text' => 'Quote gift message text',
            'wi_gift_message_from' => 'Sender',
            'wi_gift_message_to' => 'Recipient',
            'wi_items' => [
                0 => [
                    'gift_message_id' => 2,
                    'item_id' => 222,
                    'wi_gift_message_text' => 'Quote item 222 gift message text',
                    'wi_gift_message_from' => 'Quote item sender',
                    'wi_gift_message_to' => 'Quote item recipient'
                ]
            ]
        ];

        $this->assertEquals($expected, $data);
    }

    public function testProcessWithQuoteMessageAndItemMessage()
    {
        $data = [
            'gift_message_id' => 1
        ];

        $this->giftMessageProvider->expects($this->once())
            ->method('getQuoteGiftMessage')
            ->with($this->quoteId)
            ->willReturn('Quote gift message text');

        $this->cart->expects($this->once())->method('getId')->willReturn($this->quoteId);

        $giftMessages = new GiftMessages($this->giftMessageProvider);
        $giftMessages->process($this->cart, $data);

        $expected = [
            'gift_message_id' => 1,
            'wi_gift_message_text' => 'Quote gift message text',
            'wi_gift_message_from' => null,
            'wi_gift_message_to' => null
        ];

        $this->assertEquals($expected, $data);
    }

    public function testProcessWithoutMessages()
    {
        $data = [];
        $this->giftMessageProvider->expects($this->never())->method('getQuoteGiftMessage');
        $this->cart->expects($this->once())
            ->method('getId')
            ->willReturn(1);

        $giftMessages = new GiftMessages($this->giftMessageProvider);
        $giftMessages->process($this->cart, $data);

        $expected = [];

        $this->assertEquals($expected, $data);
    }
}
