<?php

namespace Glopal\Connector\Test\Unit\Processor\CartTransfer;

use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Processor\CartTransfer\TaxOptions;
use Magento\Quote\Api\Data\CartInterface;
use PHPUnit\Framework\TestCase;

// phpcs:ignoreFile
class TaxOptionsTest extends TestCase
{
    public function testProcess()
    {
        $configuration = $this->createMock(Configuration::class);

        $quote = $this->getMockForAbstractClass(
            CartInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getStoreId']
        );

        $configPathPool = [
            'price_includes_tax' => 'tax/calculation/price_includes_tax',
            'algorithm' => 'tax/calculation/algorithm',
            'apply_after_discount' => 'tax/calculation/apply_after_discount',
            'discount_tax' => 'tax/calculation/discount_tax',
            'apply_tax_on' => 'tax/calculation/apply_tax_on',
            'cross_border_trade_enabled' => 'tax/calculation/cross_border_trade_enabled',
            'tax_display_shipping' => 'tax/display/shipping',
            'tax_display_type' => 'tax/display/type',
            'tax_calculation_shipping_includes_tax' => 'tax/calculation/shipping_includes_tax',
        ];

        $taxOptions = new TaxOptions($configuration, $configPathPool);

        $storeId = 1;
        $quote->expects($this->once())->method('getStoreId')->willReturn($storeId);

        $expectedResult = [
            'wi_tax_options' => [
                'price_includes_tax' => '0',
                'algorithm' => 'TOTAL_BASE_CALCULATION',
                'apply_after_discount' => '1',
                'discount_tax' => '0',
                'apply_tax_on' => '0',
                'cross_border_trade_enabled' => '',
                'tax_display_shipping' => '1',
                'tax_display_type' => '1',
                'tax_calculation_shipping_includes_tax' => '0',
            ]
        ];

        $configuration->expects($this->exactly(9))
            ->method('getValue')
            ->withConsecutive(
                ['tax/calculation/price_includes_tax', $storeId],
                ['tax/calculation/algorithm', $storeId],
                ['tax/calculation/apply_after_discount', $storeId],
                ['tax/calculation/discount_tax', $storeId],
                ['tax/calculation/apply_tax_on', $storeId],
                ['tax/calculation/cross_border_trade_enabled', $storeId],
                ['tax/display/shipping', $storeId],
                ['tax/display/type', $storeId],
                ['tax/calculation/shipping_includes_tax', $storeId]
            )->willReturnOnConsecutiveCalls(
                '0',
                'TOTAL_BASE_CALCULATION',
                '1',
                '0',
                '0',
                '',
                '1',
                '1',
                '0'
            );

        $taxOptions->process($quote, $expectedResult);
    }
}
