<?php

namespace Glopal\Connector\Test\Unit\Processor;

use Glopal\Connector\Helper\CartTransfer as CartTransferHelper;
use Glopal\Connector\Processor\CartTransfer;
use Glopal\Connector\Processor\CartTransfer\CartTransferProcessorInterface;
use Glopal\Connector\Test\Unit\CartTransferTestHelper;
use Magento\Quote\Api\Data\CartInterface;
use PHPUnit\Framework\TestCase;
use UnexpectedValueException;

// phpcs:ignoreFile
class CartTransferTest extends TestCase
{
    public function testProcess()
    {
        $quote = $this->getMockForAbstractClass(
            CartInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getId', 'getStoreId', 'getAllVisibleItems', 'getData']
        );
        $helper = new CartTransferTestHelper();
        $cartTransferProcessor = $helper->getCartTransferProcessor($quote);
        $result = $cartTransferProcessor->process($quote);

        $expectedResult = $helper->getExpectedResult();

        $this->assertEquals($expectedResult, $result);
    }

    public function testProcessWithNotExistingQuote()
    {
        $quote = $this->getMockForAbstractClass(
            CartInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getId', 'getStoreId', 'getAllVisibleItems', 'getData']
        );
        $helper = new CartTransferTestHelper();
        $cartTransferProcessor = $helper->getCartTransferProcessorForEmptyQuote($quote);
        $result = $cartTransferProcessor->process($quote);

        $expectedResult = $helper->getExpectedResultForEmptyQuote();

        $this->assertEquals($expectedResult, $result);
    }

    public function testProcessThrowsExceptionIfProcessorDoesNotImplementInterface()
    {
        $this->expectException(UnexpectedValueException::class);
        $this->expectExceptionMessage('Processor must implement ' . CartTransferProcessorInterface::class);

        $cartTransferHelper = $this->createMock(CartTransferHelper::class);
        $cartTransfer = new CartTransfer($cartTransferHelper, [$this]);

        $quote = $this->getMockForAbstractClass(
            CartInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getData']
        );

        $cartTransfer->process($quote);
    }
}
