<?php

namespace Glopal\Connector\Test\Unit\Provider\CartTransfer;

use Glopal\Connector\Provider\CartTransfer\GiftMessageProvider;
use Magento\GiftMessage\Api\CartRepositoryInterface;
use Magento\GiftMessage\Api\Data\MessageInterface;
use Magento\GiftMessage\Api\ItemRepositoryInterface;
use PHPUnit\Framework\TestCase;

// phpcs:ignoreFile
class GiftMessageProviderTest extends TestCase
{
    private $giftMessageCartRepository;
    private $giftMessageItemRepository;

    protected function setUp(): void
    {
        $this->giftMessageCartRepository = $this->getMockForAbstractClass(
            CartRepositoryInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['get']
        );

        $this->giftMessageItemRepository = $this->getMockForAbstractClass(
            ItemRepositoryInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['get']
        );
    }

    public function testGetQuoteGiftMessage()
    {
        $provider = new GiftMessageProvider($this->giftMessageCartRepository, $this->giftMessageItemRepository);
        $messageInterfaceMock = $this->getMockForAbstractClass(MessageInterface::class);

        $this->giftMessageCartRepository->expects($this->once())
            ->method('get')
            ->with(123)
            ->willReturn($messageInterfaceMock);

        $messageInterfaceMock->expects($this->once())
            ->method('getMessage')
            ->willReturn('Quote gift message text');

        $this->assertEquals('Quote gift message text', $provider->getQuoteGiftMessage(123));
    }

    public function testGetQuoteItemGiftMessage()
    {
        $provider = new GiftMessageProvider($this->giftMessageCartRepository, $this->giftMessageItemRepository);
        $messageInterfaceMock = $this->getMockForAbstractClass(MessageInterface::class);

        $this->giftMessageItemRepository->expects($this->once())
            ->method('get')
            ->with(123, 111)
            ->willReturn($messageInterfaceMock);

        $messageInterfaceMock->expects($this->once())
            ->method('getMessage')
            ->willReturn('Quote item gift message text');

        $this->assertEquals(
            'Quote item gift message text',
            $provider->getQuoteItemGiftMessage(123, 111)
        );
    }
}
