<?php

namespace Glopal\Connector\Test\Unit\Provider\CartTransfer;

use Glopal\Connector\Provider\CartTransfer\OrderOptionsProvider;
use Magento\Catalog\Api\Data\ProductInterface;
use PHPUnit\Framework\TestCase;

// phpcs:ignoreFile
class OrderOptionsProviderTest extends TestCase
{
    private $simpleProduct;
    private $provider;

    protected function setUp(): void
    {
        $this->simpleProduct = $this->getMockForAbstractClass(
            ProductInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getId', 'getWeight']
        );

        $this->provider = new OrderOptionsProvider();
    }

    public function testGetReturnsNullWhenEmptyOrderOptionsAndProductIdIsNull()
    {
        $orderOptions = [];
        $this->simpleProduct->method('getId')->willReturn(null);

        $result = $this->provider->get($orderOptions, $this->simpleProduct);
        $this->assertNull($result);
    }

    public function testGetReturnsExpectedDataWhenEmptyOrderOptionsAndProductIdIsNotNull()
    {
        $productId = 123;
        $productWeight = 5.0;
        $orderOptions = [];
        $this->simpleProduct->method('getId')->willReturn($productId);
        $this->simpleProduct->method('getWeight')->willReturn($productWeight);

        $result = $this->provider->get($orderOptions, $this->simpleProduct);
        $this->assertEquals(['wi_variant_id' => $productId, 'wi_variant_weight' => $productWeight], $result);
    }

    public function testGetReturnsExpectedDataWhenOrderOptionsAndProductIdAreNotNull()
    {
        $productId = 123;
        $productWeight = 5.0;
        $optionId = 456;
        $optionValueId = 789;
        $orderOptions = [
            'attributes_info' => [
                [
                    'label' => 'Color',
                    'value' => 'Red',
                ],
            ],
            'info_buyRequest' => [
                'super_attribute' => [
                    $optionId => $optionValueId,
                ],
            ],
        ];

        $this->simpleProduct->method('getId')->willReturn($productId);
        $this->simpleProduct->method('getWeight')->willReturn($productWeight);

        $result = $this->provider->get($orderOptions, $this->simpleProduct);

        $this->assertEquals(
            [
                'wi_variant_id' => $productId,
                'wi_variant_weight' => $productWeight,
                'wi_attributes_merged' => [
                    [
                        'label' => 'Color',
                        'value' => 'Red',
                        'option_id' => $optionId,
                        'value_id' => $optionValueId,
                    ],
                ],
                'attributes_info' => [
                    [
                        'label' => 'Color',
                        'value' => 'Red',
                    ],
                ],
                'info_buyRequest' => [
                    'super_attribute' => [
                        $optionId => $optionValueId,
                    ],
                ],
            ],
            $result
        );
    }
}
