<?php

namespace Glopal\Connector\Test\Unit\Provider\CartTransfer;

use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Provider\CartTransfer\SmallImageProvider;
use Magento\Catalog\Model\Product\Image\UrlBuilder;
use PHPUnit\Framework\TestCase;

// phpcs:ignoreFile
class SmallImageProviderTest extends TestCase
{
    private $configurationMock;
    private $productImageUrlBuilderMock;
    private $smallImageProvider;

    protected function setUp(): void
    {
        $this->configurationMock = $this->getMockBuilder(Configuration::class)
            ->disableOriginalConstructor()
            ->setMethods(['getGeneralValue'])
            ->getMock();
        $this->productImageUrlBuilderMock = $this->createMock(UrlBuilder::class);
        $this->smallImageProvider = new SmallImageProvider($this->configurationMock, $this->productImageUrlBuilderMock);
    }

    public function testGetReturnsNullWhenPathIsEmpty()
    {
        $this->assertNull($this->smallImageProvider->get(''));
    }

    public function testGetReturnsNullWhenImageDisplayAreaIsEmpty()
    {
        $this->configurationMock
            ->expects($this->once())
            ->method('getGeneralValue')
            ->with('image_display_area')
            ->willReturn('');

        $this->assertNull($this->smallImageProvider->get('path.jpg'));
    }

    public function testGetReturnsImageWithCorrectUrl()
    {
        $imageDisplayArea = 'product_page_image_medium';
        $path = 'some-path.jpg';
        $imageUrl = 'https://example.com/media/catalog/product/cache/37fe374c0d47f738aea4523e4aa50914/w/j/path.jpg';

        $this->configurationMock
            ->expects($this->once())
            ->method('getGeneralValue')
            ->with('image_display_area')
            ->willReturn($imageDisplayArea);

        $this->productImageUrlBuilderMock
            ->expects($this->once())
            ->method('getUrl')
            ->with($path, $imageDisplayArea)
            ->willReturn($imageUrl);

        $this->assertEquals($imageUrl, $this->smallImageProvider->get($path));
    }
}
