<?php

namespace Glopal\Connector\Test\Unit\Provider\CartTransfer;

use Glopal\Connector\Helper\Configuration;
use Glopal\Connector\Provider\CartTransfer\SpecificProductAttributesProvider;
use Magento\Catalog\Api\Data\ProductInterface;
use PHPUnit\Framework\TestCase;

// phpcs:ignoreFile
class SpecificProductAttributesProviderTest extends TestCase
{
    /**
     * @dataProvider getTestData
     */
    public function testGet(
        array $attributeMapping,
        array $attributeNames,
        int $attributeSetId,
        array $productData,
        array $expectedResult
    ) {
        $configuration = $this->getMockBuilder(Configuration::class)
            ->disableOriginalConstructor()
            ->setMethods(['getCartProductDataValue'])
            ->getMock();

        $product = $this->getMockForAbstractClass(
            ProductInterface::class,
            [],
            '',
            false,
            false,
            true,
            ['getAttributeSetId', 'getData']
        );

        $provider = new SpecificProductAttributesProvider($configuration);

        $product->method('getAttributeSetId')->willReturn($attributeSetId);
        $product->method('getData')->willReturnMap($productData);

        $configuration->method('getCartProductDataValue')->willReturnMap([
            ['extra_keys_mapping', 1, $attributeMapping],
            ['extra_keys_names', 1, $attributeNames],
        ]);

        $this->assertEquals($expectedResult, $provider->get($product, 1));
    }

    public function getTestData()
    {
        return [
            'no_attributes' => [
                'attributeMapping' => [],
                'attributeNames' => [],
                'attributeSetId' => 1,
                'productData' => [],
                'expectedResult' => [],
            ],
            'attribute_in_mapping' => [
                'attributeMapping' => [
                    [
                        'glopal_key' => 'key',
                        'product_attribute_set' => 1,
                        'product_attribute' => 'some_attribute',
                    ],
                ],
                'attributeNames' => [
                    [
                        'code' => 'key',
                    ],
                ],
                'attributeSetId' => 1,
                'productData' => [
                    ['some_attribute', 'some_value'],
                ],
                'expectedResult' => [
                    'key' => 'some_value',
                ],
            ],
            'attribute_not_in_mapping' => [
                'attributeMapping' => [
                    [
                        'glopal_key' => 'key',
                        'product_attribute_set' => 1,
                        'product_attribute' => 'some_attribute',
                    ],
                ],
                'attributeNames' => [
                    [
                        'code' => 'other_key',
                    ],
                ],
                'attributeSetId' => 1,
                'productData' => [
                    ['some_attribute', 'some_value'],
                ],
                'expectedResult' => [],
            ],
            'attribute_in_different_set' => [
                'attributeMapping' => [
                    [
                        'glopal_key' => 'key',
                        'product_attribute_set' => 2,
                        'product_attribute' => 'some_attribute',
                    ],
                ],
                'attributeNames' => [
                    [
                        'code' => 'key',
                    ],
                ],
                'attributeSetId' => 1,
                'productData' => [
                    ['some_attribute', 'some_value'],
                ],
                'expectedResult' => [],
            ],
        ];
    }
}
