<?php

namespace Glopal\Connector\Validator\OrderCreate;

class Addresses extends BaseOrderCreate
{
    private const BILLING_REQUIRED_FIELDS = [
        'first_name',
        'last_name',
        'address1',
        'postcode',
        'city',
        'country_code'
    ];
    private const BILLING_OPTIONAL_FIELDS = [
        'telephone',
        'state'
    ];
    private const SHIPPING_REQUIRED_FIELDS = [
        'first_name',
        'last_name',
        'address1',
        'postcode',
        'city',
        'country_code',
        'telephone'
    ];
    private const SHIPPING_OPTIONAL_FIELDS = [
        'state'
    ];
    private const HUB_SHIPPING_REQUIRED_FIELD = [
        'first_name',
        'last_name',
        'address1',
        'postcode',
        'city',
        'country_code'
    ];
    private const HUB_SHIPPING_OPTIONAL_FIELD = [
        'state'
    ];

    /**
     * @inheritdoc
     */
    public function validate(array $data): void
    {
        // Validate Billing Address
        $billingAddressData = $data['billing_address'];
        $this->validateFieldsWithData(self::BILLING_REQUIRED_FIELDS, $billingAddressData);
        $this->validateOptionalFields(self::BILLING_OPTIONAL_FIELDS, $billingAddressData);

        // Validate Shipping Address
        $billingAddressData = $data['shipping_address'];
        $this->validateFieldsWithData(self::SHIPPING_REQUIRED_FIELDS, $billingAddressData);
        $this->validateOptionalFields(self::SHIPPING_OPTIONAL_FIELDS, $billingAddressData);

        // Validate Hub Shipping Address
        $hubAddressData = $data['shipping']['parcel_forwarding']['hub_address'] ?? null;
        if (null !== $hubAddressData) {
            $this->validateFieldsWithData(self::HUB_SHIPPING_REQUIRED_FIELD, $hubAddressData);
            $this->validateOptionalFields(self::HUB_SHIPPING_OPTIONAL_FIELD, $billingAddressData);
        }
    }
}
