<?php

namespace Glopal\Connector\Validator\OrderCreate;

use Glopal\Connector\Exception\OrderCreateValidationException;

class Items extends BaseOrderCreate
{
    private const REQUIRED_FIELDS = [
        'product_id',
        'quantity',
        'unit_price',
        'line_price',
    ];
    private const OPTIONAL_FIELDS = [
        'variant_id',
        'total_cost',
    ];
    private const TOTAL_COST_REQUIRED_FIELDS = [
        'tax_rate',
        'duty_rate',
    ];

    /**
     * @inheritdoc
     */
    public function validate(array $data): void
    {
        $data = $data['items'];

        foreach ($data as $item) {
            $key = 'items.' . key($item);
            $this->validateItem($item);
            $this->validateCharge("{$key}.unit_price", $item['unit_price']);
            $this->validateCharge("{$key}.line_price", $item['line_price']);
            $this->validateCharge(
                "{$key}.allocations.discounts.total.line_charge",
                $item['allocations']['discounts']['total']['line_charge']
            );
        }
    }

    /**
     * Validate item.
     *
     * @param array $item
     * @return void
     * @throws OrderCreateValidationException
     */
    private function validateItem(array $item): void
    {
        $this->validateFieldsWithData(self::REQUIRED_FIELDS, $item);
        $this->validateOptionalFields(self::OPTIONAL_FIELDS, $item);

        if (isset($item['total_cost'])) {
            $this->validateFieldsWithData(self::TOTAL_COST_REQUIRED_FIELDS, $item['total_cost']);
        }
    }
}
