<?php

namespace Glopal\Connector\Validator;

use Glopal\Connector\Exception\OrderCreateValidationException;
use Glopal\Connector\Validator\OrderCreate\OrderCreateValidatorInterface;
use Magento\Framework\Phrase;

class OrderCreateValidator
{
    /** @var OrderCreateValidatorInterface[] */
    private $validators;

    /**
     * @param array $validators
     */
    public function __construct(array $validators = [])
    {
        $this->validators = $validators;
    }

    /**
     * Validates order create request data.
     *
     * @param array $data
     * @return array
     */
    public function validate(array $data): array
    {
        $result = [
            'status' => true,
            'message' => ''
        ];

        try {
            foreach ($this->validators as $validator) {
                if (!$validator instanceof OrderCreateValidatorInterface) {
                    new OrderCreateValidationException(
                        new Phrase('Validator must implement ' . OrderCreateValidatorInterface::class)
                    );
                }

                $validator->validate($data);
            }
        } catch (OrderCreateValidationException $e) {
            $result['status'] = false;
            $result['message'] = $e->getMessage();

            return $result;
        }

        return $result;
    }
}
