<?php

namespace Glopal\Connector\ViewModel;

use Glopal\Connector\Api\Data\OrderAttributeInterface;
use Glopal\Connector\Api\Data\OrderTotalCostInterface;
use Glopal\Connector\Model\Order\Management;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\View\Element\Block\ArgumentInterface;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Tax\Model\Config;

class TotalsViewModel implements ArgumentInterface
{
    /** @var Management */
    private $orderManagement;

    /** @var Config */
    private $taxConfig;

    /**
     * @param Management $orderManagement
     * @param Config $taxConfig
     */
    public function __construct(Management $orderManagement, Config $taxConfig)
    {
        $this->orderManagement = $orderManagement;
        $this->taxConfig = $taxConfig;
    }

    /**
     * Display both prices?
     *
     * @return bool
     */
    public function displayBothPrices(): bool
    {
        return $this->taxConfig->displaySalesSubtotalBoth();
    }

    /**
     * Display price excluded tax?
     *
     * @return bool
     */
    public function displayPriceExclTax(): bool
    {
        return $this->taxConfig->displaySalesSubtotalExclTax();
    }

    /**
     * Display price included tax?
     *
     * @return bool
     */
    public function displayPriceInclTax(): bool
    {
        return $this->taxConfig->displaySalesSubtotalInclTax();
    }

    /**
     * Returns total cost object.
     *
     * @param OrderInterface $order
     * @return OrderTotalCostInterface|null
     * @throws NoSuchEntityException
     */
    public function getTotalCost(OrderInterface $order): ?OrderTotalCostInterface
    {
        $totalCost = $this->orderManagement->getTotalCost($order);
        if (null === $totalCost) {
            return null;
        }

        return $totalCost;
    }

    /**
     * Returns order attribute.
     *
     * @param OrderInterface $order
     * @return OrderAttributeInterface|null
     */
    public function getOrderAttributes(OrderInterface $order): ?OrderAttributeInterface
    {
        return $this->orderManagement->getOrderAttribute($order);
    }
}
