# Glopal for Magento 2

Glopal’s tailored one-stop solution covers everything from driving multi-channel traffic through Shopping Ads & International SEO, to currency conversion, translation, duty & tax calculation, international shipping and global returns.

It’s fully automated SaaS solution truly enables merchants to deliver their international strategy and scale their brand globally.

[Website](https://www.glopal.com/)

# Module description for Developers

Glopal Magento 2 module enables a connection between your Magento 2 store and Glopal services.

It exposes a set of connection endpoints under `/glopal_connector` allowing Glopal to connect with the store, synchronize products for advertising and shipping and send completed international orders back into Magento 2 store. In case there would be some server restrictions in maximum request time preventing such connection, Glopal may ask you to white-list specific requests having particular HTTP header value.

Glopal is localizing international sites with a Localization Engine solution that works as a front proxy before the Magento 2 store.
For every international visitor browsing an international site, Glopal will forward requests from Google Cloud Platform back to the store, localizing the content in real time before serving the visitor. If your Magento2 store has some DDoS protection that may block requests from Glopal, please get in touch with our support in order to setup HTTP header based white-listing allowing requests from Glopal Localization Engine.

The module will add small JavaScript content to pages in order to provide geoip features to international visitors and to ensure that international sites, once they are enabled, are properly indexed.

Domestic visitors will not encounter any changes to the domestic page once the module is installed.

# Getting started with Glopal Magento module

## Requirements:
* Technical experience and knowledge of Magento 2.
* Command line access (bash shell) to the server that is hosting your Magento installation.
* To install Glopal Magento 2 module via composer, you will need your Glopal repository token. Please write to your account manager to request the token.

After the installation, store owner will need to:
- Setup Glopal account and store activation inside module configuration
- Provide required information to Glopal Customer Care agents during an introduction call
  (need by Glopal to set up international sites, advertising campaigns and activate the store)


## Install Glopal Magento 2 module

Important: Ensure that the developer switches to the correct user before running listed commands. If they were using them before for regular maintenance purposes (plugin activation, cache flushing, reindexing, etc.) there should be no problems.
In other case they may want to consult with your hosting provider tech support for extra details.


### Option A: Install from ZIP archive

#### Step 1: Download ZIP archive

[Documentation](https://help.glopal.com/hc/en-us/articles/360007529738-How-to-install-Glopal-s-Magento-2-extension-)

Please find the ZIP file with the module source linked in the documentation.

#### Step 2: Copy files to the server

Have your developers install this Magento2 module.

Module should be extracted to "app/code/" inside Magento2 installation.
Extracting files should produce directory as "app/code/Glopal/Connector" and files like "app/code/Glopal/Connector/etc/config.xml"

### Step 3: Run setup and deployment commands

```
magento setup:upgrade
magento setup:static-content:deploy
magento deploy:mode:set production
```


### Option B: Install via composer

Glopal is using a private repository for our Magento 2 extension. The repository works in the same way as the Magento’s marketplace with the only exception that it must be added to your composer.json file before installing the package.

#### Step 1: Add the repository to your composer.json

```
composer config repositories.glopal composer https://glopal.repo.packagist.com/magento/
```

#### Step 2: Install the package

```
composer require glopal/glopal-magento2

Authentication required (glopal.repo.packagist.com):
Username: token
Password: YOUR-GLOPAL-COMPOSER-TOKEN
```

### Step 3: Run setup and deployment commands

```
magento setup:upgrade
magento setup:static-content:deploy
magento deploy:mode:set production
```

### Post Installation Check

To confirm module installation, developers can visit `/glopal_connector` endpoint.
It should return a message "Request needs to be signed by Glopal." confirming that Glopal module is installed.

After installing the module, the store owner is required to perform activation from the Magento admin dashboard
under Store > Configuration > Glopal to setup Glopal account and initiate store setup.


## Activating the store

Once the module is installed international stores can be activated for a selected store view from Magento 2 dashboard.

#### Step 1: Access Glopal menu

- Access Magento 2 dashboard as the store owner
- Open Settings
- Go to Stores > Configuration > Glopal > Settings.

If you have more than one store views, select the store view to be used as a reference when launching international sites.

All international sites will use content, catalog, price and availability of this reference store view - with Glopal localizing site language and currency.

If launching with Glopal's Marketing Solution (Google Shopping Ads), you should have products from this reference store view uploaded to your Google Merchant Center account.

#### Step 2: Connect Account
Fill in the form and create your account.
If you already have a Glopal account, click on “Connect your account” link and log in.

#### Step 3: Activate stores
Fill in the form with your plugin settings and activate your stores.

